# Generated from tamil.sbl by Snowball 3.1.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class TamilStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from tamil.sbl by Snowball 3.1.0 - https://snowballstem.org/
    '''

    B_found_vetrumai_urupu = False

    def __r_has_min_length(self):
        return len(self.current) > 4

    def __r_fix_va_start(self):
        self.bra = self.cursor
        among_var = self.find_among(TamilStemmer.a_0)
        if among_var == 0:
            return False
        self.ket = self.cursor
        self.slice_from(TamilStemmer.as_0[among_var - 1])
        return True

    def __r_fix_endings(self):
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    if not self.__r_fix_ending():
                        raise lab1()
                    continue
                except lab1: pass
                self.cursor = v_2
                break
        except lab0: pass
        self.cursor = v_1
        return True

    def __r_remove_question_prefixes(self):
        self.bra = self.cursor
        if self.cursor == self.limit or self.current[self.cursor] != "\u0B8E":
            return False
        self.cursor += 1
        if self.find_among(TamilStemmer.a_1) == 0:
            return False
        if self.cursor == self.limit or self.current[self.cursor] != "\u0BCD":
            return False
        self.cursor += 1
        self.ket = self.cursor
        self.slice_del()
        v_1 = self.cursor
        self.__r_fix_va_start()
        self.cursor = v_1
        return True

    def __r_fix_ending(self):
        if len(self.current) <= 3:
            return False
        self.limit_backward = self.cursor
        self.cursor = self.limit
        while True:
            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                among_var = self.find_among_b(TamilStemmer.a_5)
                if among_var == 0:
                    raise lab0()
                self.bra = self.cursor
                if among_var == 1:
                    self.slice_del()
                elif among_var == 2:
                    v_2 = self.limit - self.cursor
                    if self.find_among_b(TamilStemmer.a_2) == 0:
                        raise lab0()
                    self.cursor = self.limit - v_2
                    self.slice_del()
                elif among_var == 3:
                    self.slice_from("\u0BB3\u0BCD")
                elif among_var == 4:
                    self.slice_from("\u0BB2\u0BCD")
                elif among_var == 5:
                    self.slice_from("\u0B9F\u0BC1")
                elif among_var == 6:
                    if not self.B_found_vetrumai_urupu:
                        raise lab0()
                    try:
                        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u0BC8":
                            raise lab1()
                        self.cursor -= 1
                        raise lab0()
                    except lab1: pass
                    self.slice_from("\u0BAE\u0BCD")
                elif among_var == 7:
                    self.slice_from("\u0BCD")
                elif among_var == 8:
                    v_3 = self.limit - self.cursor
                    try:
                        if self.find_among_b(TamilStemmer.a_3) == 0:
                            raise lab1()
                        raise lab0()
                    except lab1: pass
                    self.cursor = self.limit - v_3
                    self.slice_del()
                else:
                    among_var = self.find_among_b(TamilStemmer.a_4)
                    if among_var == 1:
                        self.slice_del()
                    else:
                        self.slice_from("\u0BAE\u0BCD")
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            self.ket = self.cursor
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u0BCD":
                return False
            self.cursor -= 1
            while True:
                v_4 = self.limit - self.cursor
                try:
                    if self.find_among_b(TamilStemmer.a_6) == 0:
                        raise lab0()
                    v_5 = self.limit - self.cursor
                    try:
                        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u0BCD":
                            self.cursor = self.limit - v_5
                            raise lab1()
                        self.cursor -= 1
                        if self.find_among_b(TamilStemmer.a_7) == 0:
                            self.cursor = self.limit - v_5
                            raise lab1()
                    except lab1: pass
                    self.bra = self.cursor
                    self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_4
                try:
                    if self.find_among_b(TamilStemmer.a_8) == 0:
                        raise lab0()
                    self.bra = self.cursor
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u0BCD":
                        raise lab0()
                    self.cursor -= 1
                    self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_4
                v_6 = self.limit - self.cursor
                if self.find_among_b(TamilStemmer.a_9) == 0:
                    return False
                self.cursor = self.limit - v_6
                self.bra = self.cursor
                self.slice_del()
                break
            break
        self.cursor = self.limit_backward
        return True

    def __r_remove_pronoun_prefixes(self):
        self.bra = self.cursor
        if self.find_among(TamilStemmer.a_10) == 0:
            return False
        if self.find_among(TamilStemmer.a_11) == 0:
            return False
        if self.cursor == self.limit or self.current[self.cursor] != "\u0BCD":
            return False
        self.cursor += 1
        self.ket = self.cursor
        self.slice_del()
        v_1 = self.cursor
        self.__r_fix_va_start()
        self.cursor = v_1
        return True

    def __r_remove_plural_suffix(self):
        self.limit_backward = self.cursor
        self.cursor = self.limit
        self.ket = self.cursor
        among_var = self.find_among_b(TamilStemmer.a_13)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            while True:
                v_1 = self.limit - self.cursor
                try:
                    if self.find_among_b(TamilStemmer.a_12) == 0:
                        raise lab0()
                    self.slice_from("\u0BC1\u0B99\u0BCD")
                    break
                except lab0: pass
                self.cursor = self.limit - v_1
                self.slice_from("\u0BCD")
                break
        elif among_var == 2:
            self.slice_from("\u0BB2\u0BCD")
        elif among_var == 3:
            self.slice_from("\u0BB3\u0BCD")
        else:
            self.slice_del()
        self.cursor = self.limit_backward
        return True

    def __r_remove_question_suffixes(self):
        if not self.__r_has_min_length():
            return False
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if self.find_among_b(TamilStemmer.a_14) == 0:
                raise lab0()
            self.bra = self.cursor
            self.slice_from("\u0BCD")
        except lab0: pass
        self.cursor = self.limit - v_1
        self.cursor = self.limit_backward
        self.__r_fix_endings()
        return True

    def __r_remove_command_suffixes(self):
        if not self.__r_has_min_length():
            return False
        self.limit_backward = self.cursor
        self.cursor = self.limit
        self.ket = self.cursor
        if self.find_among_b(TamilStemmer.a_15) == 0:
            return False
        self.bra = self.cursor
        self.slice_del()
        self.cursor = self.limit_backward
        return True

    def __r_remove_um(self):
        if not self.__r_has_min_length():
            return False
        self.limit_backward = self.cursor
        self.cursor = self.limit
        self.ket = self.cursor
        if not self.eq_s_b("\u0BC1\u0BAE\u0BCD"):
            return False
        self.bra = self.cursor
        self.slice_from("\u0BCD")
        self.cursor = self.limit_backward
        v_1 = self.cursor
        self.__r_fix_ending()
        self.cursor = v_1
        return True

    def __r_remove_common_word_endings(self):
        if not self.__r_has_min_length():
            return False
        self.limit_backward = self.cursor
        self.cursor = self.limit
        self.ket = self.cursor
        among_var = self.find_among_b(TamilStemmer.a_17)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            self.slice_from("\u0BCD")
        elif among_var == 2:
            v_1 = self.limit - self.cursor
            try:
                if self.find_among_b(TamilStemmer.a_16) == 0:
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_1
            self.slice_from("\u0BCD")
        else:
            self.slice_del()
        self.cursor = self.limit_backward
        self.__r_fix_endings()
        return True

    def __r_remove_vetrumai_urupukal(self):
        self.B_found_vetrumai_urupu = False
        if not self.__r_has_min_length():
            return False
        self.limit_backward = self.cursor
        self.cursor = self.limit
        while True:
            v_1 = self.limit - self.cursor
            try:
                v_2 = self.limit - self.cursor
                self.ket = self.cursor
                among_var = self.find_among_b(TamilStemmer.a_20)
                if among_var == 0:
                    raise lab0()
                self.bra = self.cursor
                if among_var == 1:
                    self.slice_del()
                elif among_var == 2:
                    self.slice_from("\u0BCD")
                elif among_var == 3:
                    try:
                        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u0BAE":
                            raise lab1()
                        self.cursor -= 1
                        raise lab0()
                    except lab1: pass
                    self.slice_from("\u0BCD")
                elif among_var == 4:
                    if len(self.current) < 7:
                        raise lab0()
                    self.slice_from("\u0BCD")
                elif among_var == 5:
                    v_3 = self.limit - self.cursor
                    try:
                        if self.find_among_b(TamilStemmer.a_18) == 0:
                            raise lab1()
                        raise lab0()
                    except lab1: pass
                    self.cursor = self.limit - v_3
                    self.slice_from("\u0BCD")
                elif among_var == 6:
                    v_4 = self.limit - self.cursor
                    try:
                        if self.find_among_b(TamilStemmer.a_19) == 0:
                            raise lab1()
                        raise lab0()
                    except lab1: pass
                    self.cursor = self.limit - v_4
                    self.slice_del()
                else:
                    self.slice_from("\u0BBF")
                self.cursor = self.limit - v_2
                break
            except lab0: pass
            self.cursor = self.limit - v_1
            v_5 = self.limit - self.cursor
            self.ket = self.cursor
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u0BC8":
                return False
            self.cursor -= 1
            while True:
                v_6 = self.limit - self.cursor
                try:
                    v_7 = self.limit - self.cursor
                    try:
                        if self.find_among_b(TamilStemmer.a_21) == 0:
                            raise lab1()
                        raise lab0()
                    except lab1: pass
                    self.cursor = self.limit - v_7
                    break
                except lab0: pass
                self.cursor = self.limit - v_6
                v_8 = self.limit - self.cursor
                if self.find_among_b(TamilStemmer.a_22) == 0:
                    return False
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u0BCD":
                    return False
                self.cursor -= 1
                self.cursor = self.limit - v_8
                break
            self.bra = self.cursor
            self.slice_from("\u0BCD")
            self.cursor = self.limit - v_5
            break
        self.B_found_vetrumai_urupu = True
        v_9 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if not self.eq_s_b("\u0BBF\u0BA9\u0BCD"):
                raise lab0()
            self.bra = self.cursor
            self.slice_from("\u0BCD")
        except lab0: pass
        self.cursor = self.limit - v_9
        self.cursor = self.limit_backward
        self.__r_fix_endings()
        return True

    def __r_remove_tense_suffixes(self):
        while True:
            v_1 = self.cursor
            try:
                if not self.__r_remove_tense_suffix():
                    raise lab0()
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_remove_tense_suffix(self):
        B_found_a_match = False
        if not self.__r_has_min_length():
            return False
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_1 = self.limit - self.cursor
        try:
            v_2 = self.limit - self.cursor
            self.ket = self.cursor
            among_var = self.find_among_b(TamilStemmer.a_25)
            if among_var == 0:
                raise lab0()
            self.bra = self.cursor
            if among_var == 1:
                self.slice_del()
            elif among_var == 2:
                v_3 = self.limit - self.cursor
                try:
                    if self.find_among_b(TamilStemmer.a_23) == 0:
                        raise lab1()
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_3
                self.slice_del()
            elif among_var == 3:
                v_4 = self.limit - self.cursor
                try:
                    if self.find_among_b(TamilStemmer.a_24) == 0:
                        raise lab1()
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_4
                self.slice_del()
            elif among_var == 4:
                try:
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u0B9A":
                        raise lab1()
                    self.cursor -= 1
                    raise lab0()
                except lab1: pass
                self.slice_from("\u0BCD")
            elif among_var == 5:
                self.slice_from("\u0BCD")
            else:
                v_5 = self.limit - self.cursor
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u0BCD":
                    raise lab0()
                self.cursor -= 1
                self.cursor = self.limit - v_5
                self.slice_del()
            B_found_a_match = True
            self.cursor = self.limit - v_2
        except lab0: pass
        self.cursor = self.limit - v_1
        v_6 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if self.find_among_b(TamilStemmer.a_26) == 0:
                raise lab0()
            self.bra = self.cursor
            self.slice_del()
            B_found_a_match = True
        except lab0: pass
        self.cursor = self.limit - v_6
        self.cursor = self.limit_backward
        self.__r_fix_endings()
        return B_found_a_match

    def _stem(self):
        self.B_found_vetrumai_urupu = False
        v_1 = self.cursor
        self.__r_fix_ending()
        self.cursor = v_1
        if not self.__r_has_min_length():
            return False
        v_2 = self.cursor
        self.__r_remove_question_prefixes()
        self.cursor = v_2
        v_3 = self.cursor
        self.__r_remove_pronoun_prefixes()
        self.cursor = v_3
        self.__r_remove_question_suffixes()
        v_4 = self.cursor
        self.__r_remove_um()
        self.cursor = v_4
        v_5 = self.cursor
        self.__r_remove_common_word_endings()
        self.cursor = v_5
        v_6 = self.cursor
        self.__r_remove_vetrumai_urupukal()
        self.cursor = v_6
        v_7 = self.cursor
        self.__r_remove_plural_suffix()
        self.cursor = v_7
        v_8 = self.cursor
        self.__r_remove_command_suffixes()
        self.cursor = v_8
        v_9 = self.cursor
        self.__r_remove_tense_suffixes()
        self.cursor = v_9
        return True

    a_0 = [
        Among("\u0BB5\u0BC1", -1, 3),
        Among("\u0BB5\u0BC2", -1, 4),
        Among("\u0BB5\u0BCA", -1, 2),
        Among("\u0BB5\u0BCB", -1, 1)
    ]
    as_0 = ("\u0B93", "\u0B92", "\u0B89", "\u0B8A")

    a_1 = [
        Among("\u0B95", -1, -1),
        Among("\u0B99", -1, -1),
        Among("\u0B9A", -1, -1),
        Among("\u0B9E", -1, -1),
        Among("\u0BA4", -1, -1),
        Among("\u0BA8", -1, -1),
        Among("\u0BAA", -1, -1),
        Among("\u0BAE", -1, -1),
        Among("\u0BAF", -1, -1),
        Among("\u0BB5", -1, -1)
    ]

    a_2 = [
        Among("\u0BBF", -1, -1),
        Among("\u0BC0", -1, -1),
        Among("\u0BC8", -1, -1)
    ]

    a_3 = [
        Among("\u0BBE", -1, -1),
        Among("\u0BBF", -1, -1),
        Among("\u0BC0", -1, -1),
        Among("\u0BC1", -1, -1),
        Among("\u0BC2", -1, -1),
        Among("\u0BC6", -1, -1),
        Among("\u0BC7", -1, -1),
        Among("\u0BC8", -1, -1)
    ]

    a_4 = [
        Among("", -1, 2),
        Among("\u0BC8", 0, 1),
        Among("\u0BCD", 0, 1)
    ]

    a_5 = [
        Among("\u0BA8\u0BCD\u0BA4", -1, 1),
        Among("\u0BAF", -1, 1),
        Among("\u0BB5", -1, 1),
        Among("\u0BA9\u0BC1", -1, 8),
        Among("\u0BC1\u0B95\u0BCD", -1, 7),
        Among("\u0BC1\u0B95\u0BCD\u0B95\u0BCD", -1, 7),
        Among("\u0B9F\u0BCD\u0B95\u0BCD", -1, 3),
        Among("\u0BB1\u0BCD\u0B95\u0BCD", -1, 4),
        Among("\u0B99\u0BCD", -1, 9),
        Among("\u0B9F\u0BCD\u0B9F\u0BCD", -1, 5),
        Among("\u0BA4\u0BCD\u0BA4\u0BCD", -1, 6),
        Among("\u0BA8\u0BCD\u0BA4\u0BCD", -1, 1),
        Among("\u0BA8\u0BCD", -1, 1),
        Among("\u0B9F\u0BCD\u0BAA\u0BCD", -1, 3),
        Among("\u0BAF\u0BCD", -1, 2),
        Among("\u0BA9\u0BCD\u0BB1\u0BCD", -1, 4),
        Among("\u0BB5\u0BCD", -1, 1)
    ]

    a_6 = [
        Among("\u0B95", -1, -1),
        Among("\u0B9A", -1, -1),
        Among("\u0B9F", -1, -1),
        Among("\u0BA4", -1, -1),
        Among("\u0BAA", -1, -1),
        Among("\u0BB1", -1, -1)
    ]

    a_7 = [
        Among("\u0B95", -1, -1),
        Among("\u0B9A", -1, -1),
        Among("\u0B9F", -1, -1),
        Among("\u0BA4", -1, -1),
        Among("\u0BAA", -1, -1),
        Among("\u0BB1", -1, -1)
    ]

    a_8 = [
        Among("\u0B9E", -1, -1),
        Among("\u0BA3", -1, -1),
        Among("\u0BA8", -1, -1),
        Among("\u0BA9", -1, -1),
        Among("\u0BAE", -1, -1),
        Among("\u0BAF", -1, -1),
        Among("\u0BB0", -1, -1),
        Among("\u0BB2", -1, -1),
        Among("\u0BB3", -1, -1),
        Among("\u0BB4", -1, -1),
        Among("\u0BB5", -1, -1)
    ]

    a_9 = [
        Among("\u0BBE", -1, -1),
        Among("\u0BBF", -1, -1),
        Among("\u0BC0", -1, -1),
        Among("\u0BC1", -1, -1),
        Among("\u0BC2", -1, -1),
        Among("\u0BC6", -1, -1),
        Among("\u0BC7", -1, -1),
        Among("\u0BC8", -1, -1),
        Among("\u0BCD", -1, -1)
    ]

    a_10 = [
        Among("\u0B85", -1, -1),
        Among("\u0B87", -1, -1),
        Among("\u0B89", -1, -1)
    ]

    a_11 = [
        Among("\u0B95", -1, -1),
        Among("\u0B99", -1, -1),
        Among("\u0B9A", -1, -1),
        Among("\u0B9E", -1, -1),
        Among("\u0BA4", -1, -1),
        Among("\u0BA8", -1, -1),
        Among("\u0BAA", -1, -1),
        Among("\u0BAE", -1, -1),
        Among("\u0BAF", -1, -1),
        Among("\u0BB5", -1, -1)
    ]

    a_12 = [
        Among("\u0B95", -1, -1),
        Among("\u0B9A", -1, -1),
        Among("\u0B9F", -1, -1),
        Among("\u0BA4", -1, -1),
        Among("\u0BAA", -1, -1),
        Among("\u0BB1", -1, -1)
    ]

    a_13 = [
        Among("\u0B95\u0BB3\u0BCD", -1, 4),
        Among("\u0BC1\u0B99\u0BCD\u0B95\u0BB3\u0BCD", 0, 1),
        Among("\u0B9F\u0BCD\u0B95\u0BB3\u0BCD", 0, 3),
        Among("\u0BB1\u0BCD\u0B95\u0BB3\u0BCD", 0, 2)
    ]

    a_14 = [
        Among("\u0BBE", -1, -1),
        Among("\u0BC7", -1, -1),
        Among("\u0BCB", -1, -1)
    ]

    a_15 = [
        Among("\u0BAA\u0BBF", -1, -1),
        Among("\u0BB5\u0BBF", -1, -1)
    ]

    a_16 = [
        Among("\u0BBE", -1, -1),
        Among("\u0BBF", -1, -1),
        Among("\u0BC0", -1, -1),
        Among("\u0BC1", -1, -1),
        Among("\u0BC2", -1, -1),
        Among("\u0BC6", -1, -1),
        Among("\u0BC7", -1, -1),
        Among("\u0BC8", -1, -1)
    ]

    a_17 = [
        Among("\u0BAA\u0B9F\u0BCD\u0B9F", -1, 3),
        Among("\u0BAA\u0B9F\u0BCD\u0B9F\u0BA3", -1, 3),
        Among("\u0BA4\u0BBE\u0BA9", -1, 3),
        Among("\u0BAA\u0B9F\u0BBF\u0BA4\u0BBE\u0BA9", 2, 3),
        Among("\u0BC6\u0BA9", -1, 1),
        Among("\u0BBE\u0B95\u0BBF\u0BAF", -1, 1),
        Among("\u0B95\u0BC1\u0BB0\u0BBF\u0BAF", -1, 3),
        Among("\u0BC1\u0B9F\u0BC8\u0BAF", -1, 1),
        Among("\u0BB2\u0BCD\u0BB2", -1, 2),
        Among("\u0BC1\u0BB3\u0BCD\u0BB3", -1, 1),
        Among("\u0BBE\u0B95\u0BBF", -1, 1),
        Among("\u0BAA\u0B9F\u0BBF", -1, 3),
        Among("\u0BBF\u0BA9\u0BCD\u0BB1\u0BBF", -1, 1),
        Among("\u0BAA\u0BB1\u0BCD\u0BB1\u0BBF", -1, 3),
        Among("\u0BAA\u0B9F\u0BC1", -1, 3),
        Among("\u0BB5\u0BBF\u0B9F\u0BC1", -1, 3),
        Among("\u0BAA\u0B9F\u0BCD\u0B9F\u0BC1", -1, 3),
        Among("\u0BB5\u0BBF\u0B9F\u0BCD\u0B9F\u0BC1", -1, 3),
        Among("\u0BAA\u0B9F\u0BCD\u0B9F\u0BA4\u0BC1", -1, 3),
        Among("\u0BC6\u0BA9\u0BCD\u0BB1\u0BC1", -1, 1),
        Among("\u0BC1\u0B9F\u0BC8", -1, 1),
        Among("\u0BBF\u0BB2\u0BCD\u0BB2\u0BC8", -1, 1),
        Among("\u0BC1\u0B9F\u0BA9\u0BCD", -1, 1),
        Among("\u0BBF\u0B9F\u0BAE\u0BCD", -1, 1),
        Among("\u0BC6\u0BB2\u0BCD\u0BB2\u0BBE\u0BAE\u0BCD", -1, 3),
        Among("\u0BC6\u0BA9\u0BC1\u0BAE\u0BCD", -1, 1)
    ]

    a_18 = [
        Among("\u0BBE", -1, -1),
        Among("\u0BBF", -1, -1),
        Among("\u0BC0", -1, -1),
        Among("\u0BC1", -1, -1),
        Among("\u0BC2", -1, -1),
        Among("\u0BC6", -1, -1),
        Among("\u0BC7", -1, -1),
        Among("\u0BC8", -1, -1)
    ]

    a_19 = [
        Among("\u0BBE", -1, -1),
        Among("\u0BBF", -1, -1),
        Among("\u0BC0", -1, -1),
        Among("\u0BC1", -1, -1),
        Among("\u0BC2", -1, -1),
        Among("\u0BC6", -1, -1),
        Among("\u0BC7", -1, -1),
        Among("\u0BC8", -1, -1)
    ]

    a_20 = [
        Among("\u0BB5\u0BBF\u0B9F", -1, 2),
        Among("\u0BC0", -1, 7),
        Among("\u0BCA\u0B9F\u0BC1", -1, 2),
        Among("\u0BCB\u0B9F\u0BC1", -1, 2),
        Among("\u0BA4\u0BC1", -1, 6),
        Among("\u0BBF\u0BB0\u0BC1\u0BA8\u0BCD\u0BA4\u0BC1", 4, 2),
        Among("\u0BBF\u0BA9\u0BCD\u0BB1\u0BC1", -1, 2),
        Among("\u0BC1\u0B9F\u0BC8", -1, 2),
        Among("\u0BA9\u0BC8", -1, 1),
        Among("\u0B95\u0BA3\u0BCD", -1, 1),
        Among("\u0BBF\u0BA9\u0BCD", -1, 3),
        Among("\u0BAE\u0BC1\u0BA9\u0BCD", -1, 1),
        Among("\u0BBF\u0B9F\u0BAE\u0BCD", -1, 4),
        Among("\u0BBF\u0BB1\u0BCD", -1, 2),
        Among("\u0BAE\u0BC7\u0BB1\u0BCD", -1, 1),
        Among("\u0BB2\u0BCD", -1, 5),
        Among("\u0BBE\u0BAE\u0BB2\u0BCD", 15, 2),
        Among("\u0BBE\u0BB2\u0BCD", 15, 2),
        Among("\u0BBF\u0BB2\u0BCD", 15, 2),
        Among("\u0BAE\u0BC7\u0BB2\u0BCD", 15, 1),
        Among("\u0BC1\u0BB3\u0BCD", -1, 2),
        Among("\u0B95\u0BC0\u0BB4\u0BCD", -1, 1)
    ]

    a_21 = [
        Among("\u0B95", -1, -1),
        Among("\u0B9A", -1, -1),
        Among("\u0B9F", -1, -1),
        Among("\u0BA4", -1, -1),
        Among("\u0BAA", -1, -1),
        Among("\u0BB1", -1, -1)
    ]

    a_22 = [
        Among("\u0B95", -1, -1),
        Among("\u0B9A", -1, -1),
        Among("\u0B9F", -1, -1),
        Among("\u0BA4", -1, -1),
        Among("\u0BAA", -1, -1),
        Among("\u0BB1", -1, -1)
    ]

    a_23 = [
        Among("\u0B85", -1, -1),
        Among("\u0B86", -1, -1),
        Among("\u0B87", -1, -1),
        Among("\u0B88", -1, -1),
        Among("\u0B89", -1, -1),
        Among("\u0B8A", -1, -1),
        Among("\u0B8E", -1, -1),
        Among("\u0B8F", -1, -1),
        Among("\u0B90", -1, -1),
        Among("\u0B92", -1, -1),
        Among("\u0B93", -1, -1),
        Among("\u0B94", -1, -1)
    ]

    a_24 = [
        Among("\u0BBE", -1, -1),
        Among("\u0BBF", -1, -1),
        Among("\u0BC0", -1, -1),
        Among("\u0BC1", -1, -1),
        Among("\u0BC2", -1, -1),
        Among("\u0BC6", -1, -1),
        Among("\u0BC7", -1, -1),
        Among("\u0BC8", -1, -1)
    ]

    a_25 = [
        Among("\u0B95", -1, 1),
        Among("\u0BA4", -1, 1),
        Among("\u0BA9", -1, 1),
        Among("\u0BAA", -1, 1),
        Among("\u0BAF", -1, 1),
        Among("\u0BBE", -1, 5),
        Among("\u0B95\u0BC1", -1, 6),
        Among("\u0BAA\u0B9F\u0BC1", -1, 1),
        Among("\u0BA4\u0BC1", -1, 3),
        Among("\u0BBF\u0BB1\u0BCD\u0BB1\u0BC1", -1, 1),
        Among("\u0BA9\u0BC8", -1, 1),
        Among("\u0BB5\u0BC8", -1, 1),
        Among("\u0BA9\u0BA9\u0BCD", -1, 1),
        Among("\u0BAA\u0BA9\u0BCD", -1, 1),
        Among("\u0BB5\u0BA9\u0BCD", -1, 2),
        Among("\u0BBE\u0BA9\u0BCD", -1, 4),
        Among("\u0BA9\u0BBE\u0BA9\u0BCD", 15, 1),
        Among("\u0BAE\u0BBF\u0BA9\u0BCD", -1, 1),
        Among("\u0BA9\u0BC6\u0BA9\u0BCD", -1, 1),
        Among("\u0BC7\u0BA9\u0BCD", -1, 5),
        Among("\u0BA9\u0BAE\u0BCD", -1, 1),
        Among("\u0BAA\u0BAE\u0BCD", -1, 1),
        Among("\u0BBE\u0BAE\u0BCD", -1, 5),
        Among("\u0B95\u0BC1\u0BAE\u0BCD", -1, 1),
        Among("\u0B9F\u0BC1\u0BAE\u0BCD", -1, 5),
        Among("\u0BA4\u0BC1\u0BAE\u0BCD", -1, 1),
        Among("\u0BB1\u0BC1\u0BAE\u0BCD", -1, 1),
        Among("\u0BC6\u0BAE\u0BCD", -1, 5),
        Among("\u0BC7\u0BAE\u0BCD", -1, 5),
        Among("\u0BCB\u0BAE\u0BCD", -1, 5),
        Among("\u0BBE\u0BAF\u0BCD", -1, 5),
        Among("\u0BA9\u0BB0\u0BCD", -1, 1),
        Among("\u0BAA\u0BB0\u0BCD", -1, 1),
        Among("\u0BC0\u0BAF\u0BB0\u0BCD", -1, 5),
        Among("\u0BB5\u0BB0\u0BCD", -1, 1),
        Among("\u0BBE\u0BB0\u0BCD", -1, 5),
        Among("\u0BA9\u0BBE\u0BB0\u0BCD", 35, 1),
        Among("\u0BAE\u0BBE\u0BB0\u0BCD", 35, 1),
        Among("\u0B95\u0BCA\u0BA3\u0BCD\u0B9F\u0BBF\u0BB0\u0BCD", -1, 1),
        Among("\u0BA9\u0BBF\u0BB0\u0BCD", -1, 5),
        Among("\u0BC0\u0BB0\u0BCD", -1, 5),
        Among("\u0BA9\u0BB3\u0BCD", -1, 1),
        Among("\u0BAA\u0BB3\u0BCD", -1, 1),
        Among("\u0BB5\u0BB3\u0BCD", -1, 1),
        Among("\u0BBE\u0BB3\u0BCD", -1, 5),
        Among("\u0BA9\u0BBE\u0BB3\u0BCD", 44, 1)
    ]

    a_26 = [
        Among("\u0B95\u0BBF\u0BB1", -1, -1),
        Among("\u0B95\u0BBF\u0BA9\u0BCD\u0BB1", -1, -1),
        Among("\u0BBE\u0BA8\u0BBF\u0BA9\u0BCD\u0BB1", -1, -1),
        Among("\u0B95\u0BBF\u0BB1\u0BCD", -1, -1),
        Among("\u0B95\u0BBF\u0BA9\u0BCD\u0BB1\u0BCD", -1, -1),
        Among("\u0BBE\u0BA8\u0BBF\u0BA9\u0BCD\u0BB1\u0BCD", -1, -1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass



# Generated from basque.sbl by Snowball 3.1.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class BasqueStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from basque.sbl by Snowball 3.1.0 - https://snowballstem.org/
    '''

    g_v = {"a", "e", "i", "o", "u"}

    I_p2 = 0
    I_p1 = 0
    I_pV = 0

    def __r_mark_regions(self):
        self.I_pV = self.limit
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    if not self.in_grouping(BasqueStemmer.g_v):
                        raise lab1()
                    while True:
                        v_3 = self.cursor
                        try:
                            if not self.out_grouping(BasqueStemmer.g_v):
                                raise lab2()
                            if not self.go_out_grouping(BasqueStemmer.g_v):
                                raise lab2()
                            self.cursor += 1
                            break
                        except lab2: pass
                        self.cursor = v_3
                        if not self.in_grouping(BasqueStemmer.g_v):
                            raise lab1()
                        if not self.go_in_grouping(BasqueStemmer.g_v):
                            raise lab1()
                        self.cursor += 1
                        break
                    break
                except lab1: pass
                self.cursor = v_2
                if not self.out_grouping(BasqueStemmer.g_v):
                    raise lab0()
                while True:
                    v_4 = self.cursor
                    try:
                        if not self.out_grouping(BasqueStemmer.g_v):
                            raise lab1()
                        if not self.go_out_grouping(BasqueStemmer.g_v):
                            raise lab1()
                        self.cursor += 1
                        break
                    except lab1: pass
                    self.cursor = v_4
                    if not self.in_grouping(BasqueStemmer.g_v):
                        raise lab0()
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                    break
                break
            self.I_pV = self.cursor
        except lab0: pass
        self.cursor = v_1
        v_5 = self.cursor
        try:
            if not self.go_out_grouping(BasqueStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(BasqueStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(BasqueStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(BasqueStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_5
        return True

    def __r_RV(self):
        return self.I_pV <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_aditzak(self):
        self.ket = self.cursor
        among_var = self.find_among_b(BasqueStemmer.a_0)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_RV():
                return False
            self.slice_del()
        elif among_var == 2:
            if not self.__r_R2():
                return False
            self.slice_del()
        return True

    def __r_izenak(self):
        self.ket = self.cursor
        among_var = self.find_among_b(BasqueStemmer.a_1)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_RV():
                return False
            self.slice_del()
        elif among_var == 2:
            if not self.__r_R2():
                return False
            self.slice_del()
        elif among_var == 3:
            self.slice_from("jok")
        elif among_var == 4:
            if not self.__r_R1():
                return False
            self.slice_del()
        elif among_var == 5:
            self.slice_from("tra")
        elif among_var == 6:
            self.slice_from("minutu")
        return True

    def __r_adjetiboak(self):
        self.ket = self.cursor
        among_var = self.find_among_b(BasqueStemmer.a_2)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_RV():
                return False
            self.slice_del()
        else:
            self.slice_from("z")
        return True

    def _stem(self):
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        while True:
            v_1 = self.limit - self.cursor
            try:
                if not self.__r_aditzak():
                    raise lab0()
                continue
            except lab0: pass
            self.cursor = self.limit - v_1
            break
        while True:
            v_2 = self.limit - self.cursor
            try:
                if not self.__r_izenak():
                    raise lab0()
                continue
            except lab0: pass
            self.cursor = self.limit - v_2
            break
        v_3 = self.limit - self.cursor
        self.__r_adjetiboak()
        self.cursor = self.limit - v_3
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among("idea", -1, 1),
        Among("bidea", 0, 1),
        Among("kidea", 0, 1),
        Among("pidea", 0, 1),
        Among("kundea", -1, 1),
        Among("galea", -1, 1),
        Among("tailea", -1, 1),
        Among("tzailea", -1, 1),
        Among("gunea", -1, 1),
        Among("kunea", -1, 1),
        Among("tzaga", -1, 1),
        Among("gaia", -1, 1),
        Among("aldia", -1, 1),
        Among("taldia", 12, 1),
        Among("karia", -1, 1),
        Among("garria", -1, 2),
        Among("karria", -1, 1),
        Among("ka", -1, 1),
        Among("tzaka", 17, 1),
        Among("la", -1, 1),
        Among("mena", -1, 1),
        Among("pena", -1, 1),
        Among("kina", -1, 1),
        Among("ezina", -1, 1),
        Among("tezina", 23, 1),
        Among("kuna", -1, 1),
        Among("tuna", -1, 1),
        Among("kizuna", -1, 1),
        Among("era", -1, 1),
        Among("bera", 28, 1),
        Among("arabera", 29, -1),
        Among("kera", 28, 1),
        Among("pera", 28, 1),
        Among("orra", -1, 1),
        Among("korra", 33, 1),
        Among("dura", -1, 1),
        Among("gura", -1, 1),
        Among("kura", -1, 1),
        Among("tura", -1, 1),
        Among("eta", -1, 1),
        Among("keta", 39, 1),
        Among("gailua", -1, 1),
        Among("eza", -1, 1),
        Among("erreza", 42, 1),
        Among("tza", -1, 2),
        Among("gaitza", 44, 1),
        Among("kaitza", 44, 1),
        Among("kuntza", 44, 1),
        Among("ide", -1, 1),
        Among("bide", 48, 1),
        Among("kide", 48, 1),
        Among("pide", 48, 1),
        Among("kunde", -1, 1),
        Among("tzake", -1, 1),
        Among("tzeke", -1, 1),
        Among("le", -1, 1),
        Among("gale", 55, 1),
        Among("taile", 55, 1),
        Among("tzaile", 55, 1),
        Among("gune", -1, 1),
        Among("kune", -1, 1),
        Among("tze", -1, 1),
        Among("atze", 61, 1),
        Among("gai", -1, 1),
        Among("aldi", -1, 1),
        Among("taldi", 64, 1),
        Among("ki", -1, 1),
        Among("ari", -1, 1),
        Among("kari", 67, 1),
        Among("lari", 67, 1),
        Among("tari", 67, 1),
        Among("etari", 70, 1),
        Among("garri", -1, 2),
        Among("karri", -1, 1),
        Among("arazi", -1, 1),
        Among("tarazi", 74, 1),
        Among("an", -1, 1),
        Among("ean", 76, 1),
        Among("rean", 77, 1),
        Among("kan", 76, 1),
        Among("etan", 76, 1),
        Among("atseden", -1, -1),
        Among("men", -1, 1),
        Among("pen", -1, 1),
        Among("kin", -1, 1),
        Among("rekin", 84, 1),
        Among("ezin", -1, 1),
        Among("tezin", 86, 1),
        Among("tun", -1, 1),
        Among("kizun", -1, 1),
        Among("go", -1, 1),
        Among("ago", 90, 1),
        Among("tio", -1, 1),
        Among("dako", -1, 1),
        Among("or", -1, 1),
        Among("kor", 94, 1),
        Among("tzat", -1, 1),
        Among("du", -1, 1),
        Among("gailu", -1, 1),
        Among("tu", -1, 1),
        Among("atu", 99, 1),
        Among("aldatu", 100, 1),
        Among("tatu", 100, 1),
        Among("baditu", 99, -1),
        Among("ez", -1, 1),
        Among("errez", 104, 1),
        Among("tzez", 104, 1),
        Among("gaitz", -1, 1),
        Among("kaitz", -1, 1)
    ]

    a_1 = [
        Among("ada", -1, 1),
        Among("kada", 0, 1),
        Among("anda", -1, 1),
        Among("denda", -1, 1),
        Among("gabea", -1, 1),
        Among("kabea", -1, 1),
        Among("aldea", -1, 1),
        Among("kaldea", 6, 1),
        Among("taldea", 6, 1),
        Among("ordea", -1, 1),
        Among("zalea", -1, 1),
        Among("tzalea", 10, 1),
        Among("gilea", -1, 1),
        Among("emea", -1, 1),
        Among("kumea", -1, 1),
        Among("nea", -1, 1),
        Among("enea", 15, 1),
        Among("zionea", 15, 1),
        Among("unea", 15, 1),
        Among("gunea", 18, 1),
        Among("pea", -1, 1),
        Among("aurrea", -1, 1),
        Among("tea", -1, 1),
        Among("kotea", 22, 1),
        Among("artea", 22, 1),
        Among("ostea", 22, 1),
        Among("etxea", -1, 1),
        Among("ga", -1, 1),
        Among("anga", 27, 1),
        Among("gaia", -1, 1),
        Among("aldia", -1, 1),
        Among("taldia", 30, 1),
        Among("handia", -1, 1),
        Among("mendia", -1, 1),
        Among("geia", -1, 1),
        Among("egia", -1, 1),
        Among("degia", 35, 1),
        Among("tegia", 35, 1),
        Among("nahia", -1, 1),
        Among("ohia", -1, 1),
        Among("kia", -1, 1),
        Among("tokia", 40, 1),
        Among("oia", -1, 1),
        Among("koia", 42, 1),
        Among("aria", -1, 1),
        Among("karia", 44, 1),
        Among("laria", 44, 1),
        Among("taria", 44, 1),
        Among("eria", -1, 1),
        Among("keria", 48, 1),
        Among("teria", 48, 1),
        Among("garria", -1, 2),
        Among("larria", -1, 1),
        Among("kirria", -1, 1),
        Among("duria", -1, 1),
        Among("asia", -1, 1),
        Among("tia", -1, 1),
        Among("ezia", -1, 1),
        Among("bizia", -1, 1),
        Among("ontzia", -1, 1),
        Among("ka", -1, 1),
        Among("joka", 60, 3),
        Among("aurka", 60, -1),
        Among("ska", 60, 1),
        Among("xka", 60, 1),
        Among("zka", 60, 1),
        Among("gibela", -1, 1),
        Among("gela", -1, 1),
        Among("kaila", -1, 1),
        Among("skila", -1, 1),
        Among("tila", -1, 1),
        Among("ola", -1, 1),
        Among("na", -1, 1),
        Among("kana", 72, 1),
        Among("ena", 72, 1),
        Among("garrena", 74, 1),
        Among("gerrena", 74, 1),
        Among("urrena", 74, 1),
        Among("zaina", 72, 1),
        Among("tzaina", 78, 1),
        Among("kina", 72, 1),
        Among("mina", 72, 1),
        Among("garna", 72, 1),
        Among("una", 72, 1),
        Among("duna", 83, 1),
        Among("asuna", 83, 1),
        Among("tasuna", 85, 1),
        Among("ondoa", -1, 1),
        Among("kondoa", 87, 1),
        Among("ngoa", -1, 1),
        Among("zioa", -1, 1),
        Among("koa", -1, 1),
        Among("takoa", 91, 1),
        Among("zkoa", 91, 1),
        Among("noa", -1, 1),
        Among("zinoa", 94, 1),
        Among("aroa", -1, 1),
        Among("taroa", 96, 1),
        Among("zaroa", 96, 1),
        Among("eroa", -1, 1),
        Among("oroa", -1, 1),
        Among("osoa", -1, 1),
        Among("toa", -1, 1),
        Among("ttoa", 102, 1),
        Among("ztoa", 102, 1),
        Among("txoa", -1, 1),
        Among("tzoa", -1, 1),
        Among("ñoa", -1, 1),
        Among("ra", -1, 1),
        Among("ara", 108, 1),
        Among("dara", 109, 1),
        Among("liara", 109, 1),
        Among("tiara", 109, 1),
        Among("tara", 109, 1),
        Among("etara", 113, 1),
        Among("tzara", 109, 1),
        Among("bera", 108, 1),
        Among("kera", 108, 1),
        Among("pera", 108, 1),
        Among("ora", 108, 2),
        Among("tzarra", 108, 1),
        Among("korra", 108, 1),
        Among("tra", 108, 1),
        Among("sa", -1, 1),
        Among("osa", 123, 1),
        Among("ta", -1, 1),
        Among("eta", 125, 1),
        Among("keta", 126, 1),
        Among("sta", 125, 1),
        Among("dua", -1, 1),
        Among("mendua", 129, 1),
        Among("ordua", 129, 1),
        Among("lekua", -1, 1),
        Among("burua", -1, 1),
        Among("durua", -1, 1),
        Among("tsua", -1, 1),
        Among("tua", -1, 1),
        Among("mentua", 136, 1),
        Among("estua", 136, 1),
        Among("txua", -1, 1),
        Among("zua", -1, 1),
        Among("tzua", 140, 1),
        Among("za", -1, 1),
        Among("eza", 142, 1),
        Among("eroza", 142, 1),
        Among("tza", 142, 2),
        Among("koitza", 145, 1),
        Among("antza", 145, 1),
        Among("gintza", 145, 1),
        Among("kintza", 145, 1),
        Among("kuntza", 145, 1),
        Among("gabe", -1, 1),
        Among("kabe", -1, 1),
        Among("kide", -1, 1),
        Among("alde", -1, 1),
        Among("kalde", 154, 1),
        Among("talde", 154, 1),
        Among("orde", -1, 1),
        Among("ge", -1, 1),
        Among("zale", -1, 1),
        Among("tzale", 159, 1),
        Among("gile", -1, 1),
        Among("eme", -1, 1),
        Among("kume", -1, 1),
        Among("ne", -1, 1),
        Among("zione", 164, 1),
        Among("une", 164, 1),
        Among("gune", 166, 1),
        Among("pe", -1, 1),
        Among("aurre", -1, 1),
        Among("te", -1, 1),
        Among("kote", 170, 1),
        Among("arte", 170, 1),
        Among("oste", 170, 1),
        Among("etxe", -1, 1),
        Among("gai", -1, 1),
        Among("di", -1, 1),
        Among("aldi", 176, 1),
        Among("taldi", 177, 1),
        Among("geldi", 176, -1),
        Among("handi", 176, 1),
        Among("mendi", 176, 1),
        Among("gei", -1, 1),
        Among("egi", -1, 1),
        Among("degi", 183, 1),
        Among("tegi", 183, 1),
        Among("nahi", -1, 1),
        Among("ohi", -1, 1),
        Among("ki", -1, 1),
        Among("toki", 188, 1),
        Among("oi", -1, 1),
        Among("goi", 190, 1),
        Among("koi", 190, 1),
        Among("ari", -1, 1),
        Among("kari", 193, 1),
        Among("lari", 193, 1),
        Among("tari", 193, 1),
        Among("garri", -1, 2),
        Among("larri", -1, 1),
        Among("kirri", -1, 1),
        Among("duri", -1, 1),
        Among("asi", -1, 1),
        Among("ti", -1, 1),
        Among("ontzi", -1, 1),
        Among("ñi", -1, 1),
        Among("ak", -1, 1),
        Among("ek", -1, 1),
        Among("tarik", -1, 1),
        Among("gibel", -1, 1),
        Among("ail", -1, 1),
        Among("kail", 209, 1),
        Among("kan", -1, 1),
        Among("tan", -1, 1),
        Among("etan", 212, 1),
        Among("en", -1, 4),
        Among("ren", 214, 2),
        Among("garren", 215, 1),
        Among("gerren", 215, 1),
        Among("urren", 215, 1),
        Among("ten", 214, 4),
        Among("tzen", 214, 4),
        Among("zain", -1, 1),
        Among("tzain", 221, 1),
        Among("kin", -1, 1),
        Among("min", -1, 1),
        Among("dun", -1, 1),
        Among("asun", -1, 1),
        Among("tasun", 226, 1),
        Among("aizun", -1, 1),
        Among("ondo", -1, 1),
        Among("kondo", 229, 1),
        Among("go", -1, 1),
        Among("ngo", 231, 1),
        Among("zio", -1, 1),
        Among("ko", -1, 1),
        Among("trako", 234, 5),
        Among("tako", 234, 1),
        Among("etako", 236, 1),
        Among("eko", 234, 1),
        Among("tariko", 234, 1),
        Among("sko", 234, 1),
        Among("tuko", 234, 1),
        Among("minutuko", 241, 6),
        Among("zko", 234, 1),
        Among("no", -1, 1),
        Among("zino", 244, 1),
        Among("ro", -1, 1),
        Among("aro", 246, 1),
        Among("igaro", 247, -1),
        Among("taro", 247, 1),
        Among("zaro", 247, 1),
        Among("ero", 246, 1),
        Among("giro", 246, 1),
        Among("oro", 246, 1),
        Among("oso", -1, 1),
        Among("to", -1, 1),
        Among("tto", 255, 1),
        Among("zto", 255, 1),
        Among("txo", -1, 1),
        Among("tzo", -1, 1),
        Among("gintzo", 259, 1),
        Among("ño", -1, 1),
        Among("zp", -1, 1),
        Among("ar", -1, 1),
        Among("dar", 263, 1),
        Among("behar", 263, 1),
        Among("zehar", 263, -1),
        Among("liar", 263, 1),
        Among("tiar", 263, 1),
        Among("tar", 263, 1),
        Among("tzar", 263, 1),
        Among("or", -1, 2),
        Among("kor", 271, 1),
        Among("os", -1, 1),
        Among("ket", -1, 1),
        Among("du", -1, 1),
        Among("mendu", 275, 1),
        Among("ordu", 275, 1),
        Among("leku", -1, 1),
        Among("buru", -1, 2),
        Among("duru", -1, 1),
        Among("tsu", -1, 1),
        Among("tu", -1, 1),
        Among("tatu", 282, 4),
        Among("mentu", 282, 1),
        Among("estu", 282, 1),
        Among("txu", -1, 1),
        Among("zu", -1, 1),
        Among("tzu", 287, 1),
        Among("gintzu", 288, 1),
        Among("z", -1, 1),
        Among("ez", 290, 1),
        Among("eroz", 290, 1),
        Among("tz", 290, 1),
        Among("koitz", 293, 1)
    ]

    a_2 = [
        Among("zlea", -1, 2),
        Among("keria", -1, 1),
        Among("la", -1, 1),
        Among("era", -1, 1),
        Among("dade", -1, 1),
        Among("tade", -1, 1),
        Among("date", -1, 1),
        Among("tate", -1, 1),
        Among("gi", -1, 1),
        Among("ki", -1, 1),
        Among("ik", -1, 1),
        Among("lanik", 10, 1),
        Among("rik", 10, 1),
        Among("larik", 12, 1),
        Among("ztik", 10, 1),
        Among("go", -1, 1),
        Among("ro", -1, 1),
        Among("ero", 16, 1),
        Among("to", -1, 1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass



# Generated from norwegian.sbl by Snowball 3.1.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class NorwegianStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from norwegian.sbl by Snowball 3.1.0 - https://snowballstem.org/
    '''

    g_v = {"a", "e", "i", "o", "u", "y", "å", "æ", "ê", "ò", "ó", "ô", "ø"}

    g_s_ending = {"b", "c", "d", "f", "g", "h", "j", "l", "m", "n", "o", "p", "t", "v", "y", "z"}

    I_p1 = 0

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    while True:
                        try:
                            if self.cursor == self.limit or self.current[self.cursor] != "'":
                                raise lab2()
                            self.cursor += 1
                            break
                        except lab2: pass
                        if self.cursor >= self.limit:
                            raise lab1()
                        self.cursor += 1
                    break
                except lab1: pass
                self.cursor = v_2
                if not self.go_out_grouping(NorwegianStemmer.g_v):
                    raise lab0()
                self.cursor += 1
                if not self.go_in_grouping(NorwegianStemmer.g_v):
                    raise lab0()
                self.cursor += 1
                break
            self.I_p1 = self.cursor
        except lab0: pass
        self.cursor = v_1
        v_3 = self.cursor
        if self.cursor + 3 > self.limit:
            return False
        self.cursor += 3
        try:
            if self.I_p1 >= self.cursor:
                raise lab0()
            self.I_p1 = self.cursor
        except lab0: pass
        self.cursor = v_3
        return True

    def __r_main_suffix(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(NorwegianStemmer.a_1)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            self.slice_del()
        elif among_var == 2:
            among_var = self.find_among_b(NorwegianStemmer.a_0)
            if among_var == 1:
                self.slice_del()
        elif among_var == 3:
            while True:
                v_3 = self.limit - self.cursor
                try:
                    if not self.in_grouping_b(NorwegianStemmer.g_s_ending):
                        raise lab0()
                    break
                except lab0: pass
                self.cursor = self.limit - v_3
                try:
                    if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "r":
                        raise lab0()
                    self.cursor -= 1
                    try:
                        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "e":
                            raise lab1()
                        self.cursor -= 1
                        raise lab0()
                    except lab1: pass
                    break
                except lab0: pass
                self.cursor = self.limit - v_3
                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "k":
                    return False
                self.cursor -= 1
                if not self.out_grouping_b(NorwegianStemmer.g_v):
                    return False
                break
            self.slice_del()
        else:
            self.slice_from("er")
        return True

    def __r_consonant_pair(self):
        v_1 = self.limit - self.cursor
        if self.cursor < self.I_p1:
            return False
        v_3 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        if self.find_among_b(NorwegianStemmer.a_2) == 0:
            self.limit_backward = v_3
            return False
        self.bra = self.cursor
        self.limit_backward = v_3
        self.cursor = self.limit - v_1
        if self.cursor <= self.limit_backward:
            return False
        self.cursor -= 1
        self.bra = self.cursor
        self.slice_del()
        return True

    def __r_other_suffix(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        if self.find_among_b(NorwegianStemmer.a_3) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        self.slice_del()
        return True

    def _stem(self):
        if not self.__r_mark_regions():
            return False
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_1 = self.limit - self.cursor
        self.__r_main_suffix()
        self.cursor = self.limit - v_1
        v_2 = self.limit - self.cursor
        self.__r_consonant_pair()
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        self.__r_other_suffix()
        self.cursor = self.limit - v_3
        self.ket = self.cursor
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "'":
            return False
        self.cursor -= 1
        self.bra = self.cursor
        self.slice_del()
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among("", -1, 1),
        Among("ind", 0, -1),
        Among("kk", 0, -1),
        Among("nk", 0, -1),
        Among("amm", 0, -1),
        Among("omm", 0, -1),
        Among("kap", 0, -1),
        Among("skap", 6, 1),
        Among("pp", 0, -1),
        Among("lt", 0, -1),
        Among("ast", 0, -1),
        Among("øst", 0, -1),
        Among("v", 0, -1),
        Among("hav", 12, 1),
        Among("giv", 12, 1)
    ]

    a_1 = [
        Among("a", -1, 1),
        Among("e", -1, 1),
        Among("ede", 1, 1),
        Among("ande", 1, 1),
        Among("ende", 1, 1),
        Among("ane", 1, 1),
        Among("ene", 1, 1),
        Among("hetene", 6, 1),
        Among("erte", 1, 4),
        Among("en", -1, 1),
        Among("heten", 9, 1),
        Among("ar", -1, 1),
        Among("er", -1, 1),
        Among("heter", 12, 1),
        Among("s", -1, 3),
        Among("as", 14, 1),
        Among("es", 14, 1),
        Among("edes", 16, 1),
        Among("endes", 16, 1),
        Among("enes", 16, 1),
        Among("hetenes", 19, 1),
        Among("ens", 14, 1),
        Among("hetens", 21, 1),
        Among("ers", 14, 2),
        Among("ets", 14, 1),
        Among("et", -1, 1),
        Among("het", 25, 1),
        Among("ert", -1, 4),
        Among("ast", -1, 1)
    ]

    a_2 = [
        Among("dt", -1, -1),
        Among("vt", -1, -1)
    ]

    a_3 = [
        Among("leg", -1, 1),
        Among("eleg", 0, 1),
        Among("ig", -1, 1),
        Among("eig", 2, 1),
        Among("lig", 2, 1),
        Among("elig", 4, 1),
        Among("els", -1, 1),
        Among("lov", -1, 1),
        Among("elov", 7, 1),
        Among("slov", 7, 1),
        Among("hetslov", 9, 1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass



# Generated from yiddish.sbl by Snowball 3.1.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class YiddishStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from yiddish.sbl by Snowball 3.1.0 - https://snowballstem.org/
    '''

    g_niked = {"\u05B0", "\u05B1", "\u05B2", "\u05B3", "\u05B4", "\u05B5", "\u05B6", "\u05B7", "\u05B8", "\u05B9", "\u05BB", "\u05BC", "\u05BF", "\u05C1", "\u05C2"}

    g_vowel = {"\u05D0", "\u05D5", "\u05D9", "\u05E2", "\u05F1", "\u05F2"}

    g_consonant = {"\u05D1", "\u05D2", "\u05D3", "\u05D4", "\u05D6", "\u05D7", "\u05D8", "\u05DA", "\u05DB", "\u05DC", "\u05DD", "\u05DE", "\u05DF", "\u05E0", "\u05E1", "\u05E3", "\u05E4", "\u05E5", "\u05E6", "\u05E7", "\u05E8", "\u05E9", "\u05EA", "\u05F0"}

    I_p1 = 0

    def __r_prelude(self):
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    while True:
                        v_3 = self.cursor
                        try:
                            self.bra = self.cursor
                            among_var = self.find_among(YiddishStemmer.a_0)
                            if among_var == 0:
                                raise lab2()
                            self.ket = self.cursor
                            if among_var == 1:
                                try:
                                    if self.cursor == self.limit or self.current[self.cursor] != "\u05BC":
                                        raise lab3()
                                    self.cursor += 1
                                    raise lab2()
                                except lab3: pass
                                self.slice_from("\u05F0")
                            elif among_var == 2:
                                try:
                                    if self.cursor == self.limit or self.current[self.cursor] != "\u05B4":
                                        raise lab3()
                                    self.cursor += 1
                                    raise lab2()
                                except lab3: pass
                                self.slice_from("\u05F1")
                            elif among_var == 3:
                                try:
                                    if self.cursor == self.limit or self.current[self.cursor] != "\u05B4":
                                        raise lab3()
                                    self.cursor += 1
                                    raise lab2()
                                except lab3: pass
                                self.slice_from("\u05F2")
                            elif among_var == 4:
                                self.slice_from("\u05DB")
                            elif among_var == 5:
                                self.slice_from("\u05DE")
                            elif among_var == 6:
                                self.slice_from("\u05E0")
                            elif among_var == 7:
                                self.slice_from("\u05E4")
                            else:
                                self.slice_from("\u05E6")
                            self.cursor = v_3
                            break
                        except lab2: pass
                        self.cursor = v_3
                        if self.cursor >= self.limit:
                            raise lab1()
                        self.cursor += 1
                    continue
                except lab1: pass
                self.cursor = v_2
                break
        except lab0: pass
        self.cursor = v_1
        v_4 = self.cursor
        try:
            while True:
                v_5 = self.cursor
                try:
                    while True:
                        v_6 = self.cursor
                        try:
                            self.bra = self.cursor
                            if not self.in_grouping(YiddishStemmer.g_niked):
                                raise lab2()
                            self.ket = self.cursor
                            self.slice_del()
                            self.cursor = v_6
                            break
                        except lab2: pass
                        self.cursor = v_6
                        if self.cursor >= self.limit:
                            raise lab1()
                        self.cursor += 1
                    continue
                except lab1: pass
                self.cursor = v_5
                break
        except lab0: pass
        self.cursor = v_4
        return True

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        v_1 = self.cursor
        try:
            self.bra = self.cursor
            if not self.eq_s("\u05D2\u05E2"):
                self.cursor = v_1
                raise lab0()
            self.ket = self.cursor
            v_2 = self.cursor
            try:
                while True:
                    try:
                        if not self.eq_s("\u05DC\u05D8"):
                            raise lab2()
                        break
                    except lab2: pass
                    try:
                        if not self.eq_s("\u05D1\u05E0"):
                            raise lab2()
                        break
                    except lab2: pass
                    if self.cursor < self.limit:
                        raise lab1()
                    break
                self.cursor = v_1
                raise lab0()
            except lab1: pass
            self.cursor = v_2
            self.slice_from("GE")
        except lab0: pass
        v_3 = self.cursor
        try:
            if self.find_among(YiddishStemmer.a_1) == 0:
                self.cursor = v_3
                raise lab0()
            while True:
                v_4 = self.cursor
                try:
                    v_5 = self.cursor
                    while True:
                        try:
                            if not self.eq_s("\u05E6\u05D5\u05D2\u05E0"):
                                raise lab2()
                            break
                        except lab2: pass
                        try:
                            if not self.eq_s("\u05E6\u05D5\u05E7\u05D8"):
                                raise lab2()
                            break
                        except lab2: pass
                        if not self.eq_s("\u05E6\u05D5\u05E7\u05E0"):
                            raise lab1()
                        break
                    if self.cursor < self.limit:
                        raise lab1()
                    self.cursor = v_5
                    break
                except lab1: pass
                self.cursor = v_4
                try:
                    v_6 = self.cursor
                    if not self.eq_s("\u05D2\u05E2\u05D1\u05E0"):
                        raise lab1()
                    self.cursor = v_6
                    break
                except lab1: pass
                self.cursor = v_4
                try:
                    self.bra = self.cursor
                    if not self.eq_s("\u05D2\u05E2"):
                        raise lab1()
                    self.ket = self.cursor
                    self.slice_from("GE")
                    break
                except lab1: pass
                self.cursor = v_4
                self.bra = self.cursor
                if not self.eq_s("\u05E6\u05D5"):
                    self.cursor = v_3
                    raise lab0()
                self.ket = self.cursor
                self.slice_from("TSU")
                break
        except lab0: pass
        v_7 = self.cursor
        if self.cursor + 3 > self.limit:
            return False
        self.cursor += 3
        I_x = self.cursor
        self.cursor = v_7
        v_8 = self.cursor
        try:
            if self.find_among(YiddishStemmer.a_2) == 0:
                self.cursor = v_8
                raise lab0()
        except lab0: pass
        v_9 = self.cursor
        try:
            if not self.in_grouping(YiddishStemmer.g_consonant):
                raise lab0()
            if not self.in_grouping(YiddishStemmer.g_consonant):
                raise lab0()
            if not self.in_grouping(YiddishStemmer.g_consonant):
                raise lab0()
            self.I_p1 = self.cursor
            return False
        except lab0: pass
        self.cursor = v_9
        if not self.go_out_grouping(YiddishStemmer.g_vowel):
            return False
        self.cursor += 1
        if not self.go_in_grouping(YiddishStemmer.g_vowel):
            return False
        self.I_p1 = self.cursor
        try:
            if self.I_p1 >= I_x:
                raise lab0()
            self.I_p1 = I_x
        except lab0: pass
        return True

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_R1plus3(self):
        return self.I_p1 <= (self.cursor + 3)

    def __r_standard_suffix(self):
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(YiddishStemmer.a_4)
            if among_var == 0:
                raise lab0()
            self.bra = self.cursor
            if among_var == 1:
                if not self.__r_R1():
                    raise lab0()
                self.slice_del()
            elif among_var == 2:
                if not self.__r_R1():
                    raise lab0()
                self.slice_from("\u05D9\u05E2")
            elif among_var == 3:
                if not self.__r_R1():
                    raise lab0()
                self.slice_del()
                self.ket = self.cursor
                among_var = self.find_among_b(YiddishStemmer.a_3)
                if among_var == 0:
                    raise lab0()
                self.bra = self.cursor
                self.slice_from(YiddishStemmer.as_3[among_var - 1])
            elif among_var == 4:
                while True:
                    v_2 = self.limit - self.cursor
                    try:
                        if not self.__r_R1():
                            raise lab1()
                        self.slice_del()
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_2
                    self.slice_from("\u05D8")
                    break
                self.ket = self.cursor
                if not self.eq_s_b("\u05D1\u05E8\u05D0\u05DB"):
                    raise lab0()
                v_3 = self.limit - self.cursor
                try:
                    if not self.eq_s_b("\u05D2\u05E2"):
                        self.cursor = self.limit - v_3
                        raise lab1()
                except lab1: pass
                self.bra = self.cursor
                self.slice_from("\u05D1\u05E8\u05E2\u05E0\u05D2")
            elif among_var == 5:
                self.slice_from("\u05D2\u05F2")
            elif among_var == 6:
                self.slice_from("\u05E0\u05E2\u05DE")
            elif among_var == 7:
                self.slice_from("\u05E9\u05E8\u05F2\u05D1")
            elif among_var == 8:
                self.slice_from("\u05DE\u05F2\u05D3")
            elif among_var == 9:
                self.slice_from("\u05D1\u05F2\u05D8")
            elif among_var == 10:
                self.slice_from("\u05D1\u05F2\u05E1")
            elif among_var == 11:
                self.slice_from("\u05F0\u05F2\u05D6")
            elif among_var == 12:
                self.slice_from("\u05D8\u05E8\u05F2\u05D1")
            elif among_var == 13:
                self.slice_from("\u05DC\u05F2\u05D8")
            elif among_var == 14:
                self.slice_from("\u05E7\u05DC\u05F2\u05D1")
            elif among_var == 15:
                self.slice_from("\u05E8\u05F2\u05D1")
            elif among_var == 16:
                self.slice_from("\u05E8\u05F2\u05E1")
            elif among_var == 17:
                self.slice_from("\u05E9\u05F0\u05F2\u05D2")
            elif among_var == 18:
                self.slice_from("\u05E9\u05DE\u05F2\u05E1")
            elif among_var == 19:
                self.slice_from("\u05E9\u05E0\u05F2\u05D3")
            elif among_var == 20:
                self.slice_from("\u05D1\u05D9\u05E0\u05D3")
            elif among_var == 21:
                self.slice_from("\u05F0\u05D9\u05D8\u05E9")
            elif among_var == 22:
                self.slice_from("\u05D6\u05D9\u05E0\u05D2")
            elif among_var == 23:
                self.slice_from("\u05D8\u05E8\u05D9\u05E0\u05E7")
            elif among_var == 24:
                self.slice_from("\u05E6\u05F0\u05D9\u05E0\u05D2")
            elif among_var == 25:
                self.slice_from("\u05E9\u05DC\u05D9\u05E0\u05D2")
            elif among_var == 26:
                self.slice_from("\u05D1\u05F2\u05D2")
            elif among_var == 27:
                self.slice_from("\u05D4\u05F2\u05D1")
            elif among_var == 28:
                self.slice_from("\u05E4\u05D0\u05E8\u05DC\u05D9\u05E8")
            elif among_var == 29:
                self.slice_from("\u05E9\u05D8\u05F2")
            elif among_var == 30:
                self.slice_from("\u05E9\u05F0\u05E2\u05E8")
            elif among_var == 31:
                self.slice_from("\u05D1\u05E8\u05E2\u05E0\u05D2")
            elif among_var == 32:
                if not self.__r_R1():
                    raise lab0()
                self.slice_from("\u05D4")
            elif among_var == 33:
                while True:
                    v_4 = self.limit - self.cursor
                    try:
                        while True:
                            try:
                                if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u05D2":
                                    raise lab2()
                                self.cursor -= 1
                                break
                            except lab2: pass
                            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "\u05E9":
                                raise lab1()
                            self.cursor -= 1
                            break
                        v_5 = self.limit - self.cursor
                        try:
                            if not self.__r_R1plus3():
                                self.cursor = self.limit - v_5
                                raise lab2()
                            self.slice_from("\u05D9\u05E1")
                        except lab2: pass
                        break
                    except lab1: pass
                    self.cursor = self.limit - v_4
                    if not self.__r_R1():
                        raise lab0()
                    self.slice_del()
                    break
        except lab0: pass
        self.cursor = self.limit - v_1
        v_6 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(YiddishStemmer.a_5)
            if among_var == 0:
                raise lab0()
            self.bra = self.cursor
            if among_var == 1:
                if not self.__r_R1():
                    raise lab0()
                self.slice_del()
            else:
                if not self.__r_R1():
                    raise lab0()
                if not self.in_grouping_b(YiddishStemmer.g_consonant):
                    raise lab0()
                self.slice_del()
        except lab0: pass
        self.cursor = self.limit - v_6
        v_7 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            among_var = self.find_among_b(YiddishStemmer.a_6)
            if among_var == 0:
                raise lab0()
            self.bra = self.cursor
            if among_var == 1:
                if not self.__r_R1():
                    raise lab0()
                self.slice_del()
        except lab0: pass
        self.cursor = self.limit - v_7
        v_8 = self.limit - self.cursor
        try:
            while True:
                v_9 = self.limit - self.cursor
                try:
                    while True:
                        v_10 = self.limit - self.cursor
                        try:
                            self.ket = self.cursor
                            while True:
                                try:
                                    if not self.eq_s_b("GE"):
                                        raise lab3()
                                    break
                                except lab3: pass
                                if not self.eq_s_b("TSU"):
                                    raise lab2()
                                break
                            self.bra = self.cursor
                            self.slice_del()
                            self.cursor = self.limit - v_10
                            break
                        except lab2: pass
                        self.cursor = self.limit - v_10
                        if self.cursor <= self.limit_backward:
                            raise lab1()
                        self.cursor -= 1
                    continue
                except lab1: pass
                self.cursor = self.limit - v_9
                break
        except lab0: pass
        self.cursor = self.limit - v_8
        return True

    def _stem(self):
        self.__r_prelude()
        v_1 = self.cursor
        self.__r_mark_regions()
        self.cursor = v_1
        self.limit_backward = self.cursor
        self.cursor = self.limit
        self.__r_standard_suffix()
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among("\u05D5\u05D5", -1, 1),
        Among("\u05D5\u05D9", -1, 2),
        Among("\u05D9\u05D9", -1, 3),
        Among("\u05DA", -1, 4),
        Among("\u05DD", -1, 5),
        Among("\u05DF", -1, 6),
        Among("\u05E3", -1, 7),
        Among("\u05E5", -1, 8)
    ]

    a_1 = [
        Among("\u05D0\u05D3\u05D5\u05E8\u05DB", -1, 1),
        Among("\u05D0\u05D4\u05D9\u05E0", -1, 1),
        Among("\u05D0\u05D4\u05E2\u05E8", -1, 1),
        Among("\u05D0\u05D4\u05F2\u05DE", -1, 1),
        Among("\u05D0\u05D5\u05DE", -1, 1),
        Among("\u05D0\u05D5\u05E0\u05D8\u05E2\u05E8", -1, 1),
        Among("\u05D0\u05D9\u05D1\u05E2\u05E8", -1, 1),
        Among("\u05D0\u05E0", -1, 1),
        Among("\u05D0\u05E0\u05D8", 7, 1),
        Among("\u05D0\u05E0\u05D8\u05E7\u05E2\u05D2\u05E0", 8, 1),
        Among("\u05D0\u05E0\u05D9\u05D3\u05E2\u05E8", 7, 1),
        Among("\u05D0\u05E4", -1, 1),
        Among("\u05D0\u05E4\u05D9\u05E8", 11, 1),
        Among("\u05D0\u05E7\u05E2\u05D2\u05E0", -1, 1),
        Among("\u05D0\u05E8\u05D0\u05E4", -1, 1),
        Among("\u05D0\u05E8\u05D5\u05DE", -1, 1),
        Among("\u05D0\u05E8\u05D5\u05E0\u05D8\u05E2\u05E8", -1, 1),
        Among("\u05D0\u05E8\u05D9\u05D1\u05E2\u05E8", -1, 1),
        Among("\u05D0\u05E8\u05F1\u05E1", -1, 1),
        Among("\u05D0\u05E8\u05F1\u05E4", -1, 1),
        Among("\u05D0\u05E8\u05F2\u05E0", -1, 1),
        Among("\u05D0\u05F0\u05E2\u05E7", -1, 1),
        Among("\u05D0\u05F1\u05E1", -1, 1),
        Among("\u05D0\u05F1\u05E4", -1, 1),
        Among("\u05D0\u05F2\u05E0", -1, 1),
        Among("\u05D1\u05D0", -1, 1),
        Among("\u05D1\u05F2", -1, 1),
        Among("\u05D3\u05D5\u05E8\u05DB", -1, 1),
        Among("\u05D3\u05E2\u05E8", -1, 1),
        Among("\u05DE\u05D9\u05D8", -1, 1),
        Among("\u05E0\u05D0\u05DB", -1, 1),
        Among("\u05E4\u05D0\u05E8", -1, 1),
        Among("\u05E4\u05D0\u05E8\u05D1\u05F2", 31, 1),
        Among("\u05E4\u05D0\u05E8\u05F1\u05E1", 31, 1),
        Among("\u05E4\u05D5\u05E0\u05D0\u05E0\u05D3\u05E2\u05E8", -1, 1),
        Among("\u05E6\u05D5", -1, 1),
        Among("\u05E6\u05D5\u05D6\u05D0\u05DE\u05E2\u05E0", 35, 1),
        Among("\u05E6\u05D5\u05E0\u05F1\u05E4", 35, 1),
        Among("\u05E6\u05D5\u05E8\u05D9\u05E7", 35, 1),
        Among("\u05E6\u05E2", -1, 1)
    ]

    a_2 = [
        Among("\u05D3\u05D6\u05E9", -1, -1),
        Among("\u05E9\u05D8\u05E8", -1, -1),
        Among("\u05E9\u05D8\u05E9", -1, -1),
        Among("\u05E9\u05E4\u05E8", -1, -1)
    ]

    a_3 = [
        Among("\u05E7\u05DC\u05D9\u05D1", -1, 9),
        Among("\u05E8\u05D9\u05D1", -1, 10),
        Among("\u05D8\u05E8\u05D9\u05D1", 1, 7),
        Among("\u05E9\u05E8\u05D9\u05D1", 1, 15),
        Among("\u05D4\u05F1\u05D1", -1, 23),
        Among("\u05E9\u05F0\u05D9\u05D2", -1, 12),
        Among("\u05D2\u05D0\u05E0\u05D2", -1, 1),
        Among("\u05D6\u05D5\u05E0\u05D2", -1, 18),
        Among("\u05E9\u05DC\u05D5\u05E0\u05D2", -1, 21),
        Among("\u05E6\u05F0\u05D5\u05E0\u05D2", -1, 20),
        Among("\u05D1\u05F1\u05D2", -1, 22),
        Among("\u05D1\u05D5\u05E0\u05D3", -1, 16),
        Among("\u05F0\u05D9\u05D6", -1, 6),
        Among("\u05D1\u05D9\u05D8", -1, 4),
        Among("\u05DC\u05D9\u05D8", -1, 8),
        Among("\u05DE\u05D9\u05D8", -1, 3),
        Among("\u05E9\u05E0\u05D9\u05D8", -1, 14),
        Among("\u05E0\u05D5\u05DE", -1, 2),
        Among("\u05E9\u05D8\u05D0\u05E0", -1, 25),
        Among("\u05D1\u05D9\u05E1", -1, 5),
        Among("\u05E9\u05DE\u05D9\u05E1", -1, 13),
        Among("\u05E8\u05D9\u05E1", -1, 11),
        Among("\u05D8\u05E8\u05D5\u05E0\u05E7", -1, 19),
        Among("\u05E4\u05D0\u05E8\u05DC\u05F1\u05E8", -1, 24),
        Among("\u05E9\u05F0\u05F1\u05E8", -1, 26),
        Among("\u05F0\u05D5\u05D8\u05E9", -1, 17)
    ]
    as_3 = ("\u05D2\u05F2", "\u05E0\u05E2\u05DE", "\u05DE\u05F2\u05D3", "\u05D1\u05F2\u05D8", "\u05D1\u05F2\u05E1", "\u05F0\u05F2\u05D6", "\u05D8\u05E8\u05F2\u05D1", "\u05DC\u05F2\u05D8", "\u05E7\u05DC\u05F2\u05D1", "\u05E8\u05F2\u05D1", "\u05E8\u05F2\u05E1", "\u05E9\u05F0\u05F2\u05D2", "\u05E9\u05DE\u05F2\u05E1", "\u05E9\u05E0\u05F2\u05D3", "\u05E9\u05E8\u05F2\u05D1", "\u05D1\u05D9\u05E0\u05D3", "\u05F0\u05D9\u05D8\u05E9", "\u05D6\u05D9\u05E0\u05D2", "\u05D8\u05E8\u05D9\u05E0\u05E7", "\u05E6\u05F0\u05D9\u05E0\u05D2", "\u05E9\u05DC\u05D9\u05E0\u05D2", "\u05D1\u05F2\u05D2", "\u05D4\u05F2\u05D1", "\u05E4\u05D0\u05E8\u05DC\u05D9\u05E8", "\u05E9\u05D8\u05F2", "\u05E9\u05F0\u05E2\u05E8")

    a_4 = [
        Among("\u05D5\u05E0\u05D2", -1, 1),
        Among("\u05E1\u05D8\u05D5", -1, 1),
        Among("\u05D8", -1, 1),
        Among("\u05D1\u05E8\u05D0\u05DB\u05D8", 2, 31),
        Among("\u05E1\u05D8", 2, 1),
        Among("\u05D9\u05E1\u05D8", 4, 33),
        Among("\u05E2\u05D8", 2, 1),
        Among("\u05E9\u05D0\u05E4\u05D8", 2, 1),
        Among("\u05D4\u05F2\u05D8", 2, 1),
        Among("\u05E7\u05F2\u05D8", 2, 1),
        Among("\u05D9\u05E7\u05F2\u05D8", 9, 1),
        Among("\u05DC\u05E2\u05DB", -1, 1),
        Among("\u05E2\u05DC\u05E2\u05DB", 11, 1),
        Among("\u05D9\u05D6\u05DE", -1, 1),
        Among("\u05D9\u05DE", -1, 1),
        Among("\u05E2\u05DE", -1, 1),
        Among("\u05E2\u05E0\u05E2\u05DE", 15, 3),
        Among("\u05D8\u05E2\u05E0\u05E2\u05DE", 16, 4),
        Among("\u05E0", -1, 1),
        Among("\u05E7\u05DC\u05D9\u05D1\u05E0", 18, 14),
        Among("\u05E8\u05D9\u05D1\u05E0", 18, 15),
        Among("\u05D8\u05E8\u05D9\u05D1\u05E0", 20, 12),
        Among("\u05E9\u05E8\u05D9\u05D1\u05E0", 20, 7),
        Among("\u05D4\u05F1\u05D1\u05E0", 18, 27),
        Among("\u05E9\u05F0\u05D9\u05D2\u05E0", 18, 17),
        Among("\u05D6\u05D5\u05E0\u05D2\u05E0", 18, 22),
        Among("\u05E9\u05DC\u05D5\u05E0\u05D2\u05E0", 18, 25),
        Among("\u05E6\u05F0\u05D5\u05E0\u05D2\u05E0", 18, 24),
        Among("\u05D1\u05F1\u05D2\u05E0", 18, 26),
        Among("\u05D1\u05D5\u05E0\u05D3\u05E0", 18, 20),
        Among("\u05F0\u05D9\u05D6\u05E0", 18, 11),
        Among("\u05D8\u05E0", 18, 4),
        Among("GE\u05D1\u05D9\u05D8\u05E0", 31, 9),
        Among("GE\u05DC\u05D9\u05D8\u05E0", 31, 13),
        Among("GE\u05DE\u05D9\u05D8\u05E0", 31, 8),
        Among("\u05E9\u05E0\u05D9\u05D8\u05E0", 31, 19),
        Among("\u05E1\u05D8\u05E0", 31, 1),
        Among("\u05D9\u05E1\u05D8\u05E0", 36, 1),
        Among("\u05E2\u05D8\u05E0", 31, 1),
        Among("GE\u05D1\u05D9\u05E1\u05E0", 18, 10),
        Among("\u05E9\u05DE\u05D9\u05E1\u05E0", 18, 18),
        Among("GE\u05E8\u05D9\u05E1\u05E0", 18, 16),
        Among("\u05E2\u05E0", 18, 1),
        Among("\u05D2\u05D0\u05E0\u05D2\u05E2\u05E0", 42, 5),
        Among("\u05E2\u05DC\u05E2\u05E0", 42, 1),
        Among("\u05E0\u05D5\u05DE\u05E2\u05E0", 42, 6),
        Among("\u05D9\u05D6\u05DE\u05E2\u05E0", 42, 1),
        Among("\u05E9\u05D8\u05D0\u05E0\u05E2\u05E0", 42, 29),
        Among("\u05D8\u05E8\u05D5\u05E0\u05E7\u05E0", 18, 23),
        Among("\u05E4\u05D0\u05E8\u05DC\u05F1\u05E8\u05E0", 18, 28),
        Among("\u05E9\u05F0\u05F1\u05E8\u05E0", 18, 30),
        Among("\u05F0\u05D5\u05D8\u05E9\u05E0", 18, 21),
        Among("\u05D2\u05F2\u05E0", 18, 5),
        Among("\u05E1", -1, 1),
        Among("\u05D8\u05E1", 53, 4),
        Among("\u05E2\u05D8\u05E1", 54, 1),
        Among("\u05E0\u05E1", 53, 1),
        Among("\u05D8\u05E0\u05E1", 56, 4),
        Among("\u05E2\u05E0\u05E1", 56, 3),
        Among("\u05E2\u05E1", 53, 1),
        Among("\u05D9\u05E2\u05E1", 59, 2),
        Among("\u05E2\u05DC\u05E2\u05E1", 59, 1),
        Among("\u05E2\u05E8\u05E1", 53, 1),
        Among("\u05E2\u05E0\u05E2\u05E8\u05E1", 62, 1),
        Among("\u05E2", -1, 1),
        Among("\u05D8\u05E2", 64, 4),
        Among("\u05E1\u05D8\u05E2", 65, 1),
        Among("\u05E2\u05D8\u05E2", 65, 1),
        Among("\u05D9\u05E2", 64, -1),
        Among("\u05E2\u05DC\u05E2", 64, 1),
        Among("\u05E2\u05E0\u05E2", 64, 3),
        Among("\u05D8\u05E2\u05E0\u05E2", 70, 4),
        Among("\u05E2\u05E8", -1, 1),
        Among("\u05D8\u05E2\u05E8", 72, 4),
        Among("\u05E1\u05D8\u05E2\u05E8", 73, 1),
        Among("\u05E2\u05D8\u05E2\u05E8", 73, 1),
        Among("\u05E2\u05E0\u05E2\u05E8", 72, 3),
        Among("\u05D8\u05E2\u05E0\u05E2\u05E8", 76, 4),
        Among("\u05D5\u05EA", -1, 32)
    ]

    a_5 = [
        Among("\u05D5\u05E0\u05D2", -1, 1),
        Among("\u05E9\u05D0\u05E4\u05D8", -1, 1),
        Among("\u05D4\u05F2\u05D8", -1, 1),
        Among("\u05E7\u05F2\u05D8", -1, 1),
        Among("\u05D9\u05E7\u05F2\u05D8", 3, 1),
        Among("\u05DC", -1, 2)
    ]

    a_6 = [
        Among("\u05D9\u05D2", -1, 1),
        Among("\u05D9\u05E7", -1, 1),
        Among("\u05D3\u05D9\u05E7", 1, 1),
        Among("\u05E0\u05D3\u05D9\u05E7", 2, 1),
        Among("\u05E2\u05E0\u05D3\u05D9\u05E7", 3, 1),
        Among("\u05D1\u05DC\u05D9\u05E7", 1, -1),
        Among("\u05D2\u05DC\u05D9\u05E7", 1, -1),
        Among("\u05E0\u05D9\u05E7", 1, 1),
        Among("\u05D9\u05E9", -1, 1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass


class lab3(BaseException): pass



# Generated from italian.sbl by Snowball 3.1.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class ItalianStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from italian.sbl by Snowball 3.1.0 - https://snowballstem.org/
    '''

    g_v = {"a", "e", "i", "o", "u", "à", "è", "ì", "ò", "ù"}

    g_AEIO = {"a", "e", "i", "o", "à", "è", "ì", "ò"}

    g_CG = "cg"
    I_p2 = 0
    I_p1 = 0
    I_pV = 0

    def __r_elisions(self):
        self.bra = self.cursor
        if self.find_among(ItalianStemmer.a_0) == 0:
            return False
        self.ket = self.cursor
        try:
            if self.cursor < self.limit:
                raise lab0()
            return False
        except lab0: pass
        self.slice_del()
        return True

    def __r_prelude(self):
        v_1 = self.cursor
        while True:
            v_2 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(ItalianStemmer.a_1)
                self.ket = self.cursor
                if among_var == 1:
                    self.slice_from("à")
                elif among_var == 2:
                    self.slice_from("è")
                elif among_var == 3:
                    self.slice_from("ì")
                elif among_var == 4:
                    self.slice_from("ò")
                elif among_var == 5:
                    self.slice_from("ù")
                elif among_var == 6:
                    self.slice_from("qU")
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_2
            break
        self.cursor = v_1
        while True:
            v_3 = self.cursor
            try:
                while True:
                    v_4 = self.cursor
                    try:
                        if not self.in_grouping(ItalianStemmer.g_v):
                            raise lab1()
                        self.bra = self.cursor
                        while True:
                            v_5 = self.cursor
                            try:
                                if self.cursor == self.limit or self.current[self.cursor] != "u":
                                    raise lab2()
                                self.cursor += 1
                                self.ket = self.cursor
                                if not self.in_grouping(ItalianStemmer.g_v):
                                    raise lab2()
                                self.slice_from("U")
                                break
                            except lab2: pass
                            self.cursor = v_5
                            if self.cursor == self.limit or self.current[self.cursor] != "i":
                                raise lab1()
                            self.cursor += 1
                            self.ket = self.cursor
                            if not self.in_grouping(ItalianStemmer.g_v):
                                raise lab1()
                            self.slice_from("I")
                            break
                        self.cursor = v_4
                        break
                    except lab1: pass
                    self.cursor = v_4
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_3
            break
        return True

    def __r_mark_regions(self):
        self.I_pV = self.limit
        self.I_p1 = self.limit
        self.I_p2 = self.limit
        v_1 = self.cursor
        try:
            while True:
                v_2 = self.cursor
                try:
                    if not self.in_grouping(ItalianStemmer.g_v):
                        raise lab1()
                    while True:
                        v_3 = self.cursor
                        try:
                            if not self.out_grouping(ItalianStemmer.g_v):
                                raise lab2()
                            if not self.go_out_grouping(ItalianStemmer.g_v):
                                raise lab2()
                            self.cursor += 1
                            break
                        except lab2: pass
                        self.cursor = v_3
                        if not self.in_grouping(ItalianStemmer.g_v):
                            raise lab1()
                        if not self.go_in_grouping(ItalianStemmer.g_v):
                            raise lab1()
                        self.cursor += 1
                        break
                    break
                except lab1: pass
                self.cursor = v_2
                try:
                    if not self.eq_s("divan"):
                        raise lab1()
                    break
                except lab1: pass
                self.cursor = v_2
                if not self.out_grouping(ItalianStemmer.g_v):
                    raise lab0()
                while True:
                    v_4 = self.cursor
                    try:
                        if not self.out_grouping(ItalianStemmer.g_v):
                            raise lab1()
                        if not self.go_out_grouping(ItalianStemmer.g_v):
                            raise lab1()
                        self.cursor += 1
                        break
                    except lab1: pass
                    self.cursor = v_4
                    if not self.in_grouping(ItalianStemmer.g_v):
                        raise lab0()
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                    break
                break
            self.I_pV = self.cursor
        except lab0: pass
        self.cursor = v_1
        v_5 = self.cursor
        try:
            if not self.go_out_grouping(ItalianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(ItalianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p1 = self.cursor
            if not self.go_out_grouping(ItalianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            if not self.go_in_grouping(ItalianStemmer.g_v):
                raise lab0()
            self.cursor += 1
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_5
        return True

    def __r_postlude(self):
        while True:
            v_1 = self.cursor
            try:
                self.bra = self.cursor
                among_var = self.find_among(ItalianStemmer.a_2)
                self.ket = self.cursor
                if among_var == 1:
                    self.slice_from("i")
                elif among_var == 2:
                    self.slice_from("u")
                else:
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                continue
            except lab0: pass
            self.cursor = v_1
            break
        return True

    def __r_RV(self):
        return self.I_pV <= self.cursor

    def __r_R1(self):
        return self.I_p1 <= self.cursor

    def __r_R2(self):
        return self.I_p2 <= self.cursor

    def __r_attached_pronoun(self):
        self.ket = self.cursor
        if self.find_among_b(ItalianStemmer.a_3) == 0:
            return False
        self.bra = self.cursor
        among_var = self.find_among_b(ItalianStemmer.a_4)
        if among_var == 0:
            return False
        if not self.__r_RV():
            return False
        if among_var == 1:
            self.slice_del()
        else:
            self.slice_from("e")
        return True

    def __r_standard_suffix(self):
        self.ket = self.cursor
        among_var = self.find_among_b(ItalianStemmer.a_7)
        if among_var == 0:
            return False
        self.bra = self.cursor
        if among_var == 1:
            if not self.__r_R2():
                return False
            self.slice_del()
        elif among_var == 2:
            if not self.__r_R2():
                return False
            self.slice_del()
            v_1 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b("ic"):
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_1
                    raise lab0()
                self.slice_del()
            except lab0: pass
        elif among_var == 3:
            if not self.__r_R2():
                return False
            self.slice_from("log")
        elif among_var == 4:
            if not self.__r_R2():
                return False
            self.slice_from("u")
        elif among_var == 5:
            if not self.__r_R2():
                return False
            self.slice_from("ente")
        elif among_var == 6:
            if not self.__r_RV():
                return False
            self.slice_del()
        elif among_var == 7:
            if not self.__r_R1():
                return False
            self.slice_del()
            v_2 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                among_var = self.find_among_b(ItalianStemmer.a_5)
                if among_var == 0:
                    self.cursor = self.limit - v_2
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_2
                    raise lab0()
                self.slice_del()
                if among_var == 1:
                    self.ket = self.cursor
                    if not self.eq_s_b("at"):
                        self.cursor = self.limit - v_2
                        raise lab0()
                    self.bra = self.cursor
                    if not self.__r_R2():
                        self.cursor = self.limit - v_2
                        raise lab0()
                    self.slice_del()
            except lab0: pass
        elif among_var == 8:
            if not self.__r_R2():
                return False
            self.slice_del()
            v_3 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if self.find_among_b(ItalianStemmer.a_6) == 0:
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_3
                    raise lab0()
                self.slice_del()
            except lab0: pass
        else:
            if not self.__r_R2():
                return False
            self.slice_del()
            v_4 = self.limit - self.cursor
            try:
                self.ket = self.cursor
                if not self.eq_s_b("at"):
                    self.cursor = self.limit - v_4
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_4
                    raise lab0()
                self.slice_del()
                self.ket = self.cursor
                if not self.eq_s_b("ic"):
                    self.cursor = self.limit - v_4
                    raise lab0()
                self.bra = self.cursor
                if not self.__r_R2():
                    self.cursor = self.limit - v_4
                    raise lab0()
                self.slice_del()
            except lab0: pass
        return True

    def __r_verb_suffix(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        if self.find_among_b(ItalianStemmer.a_8) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.slice_del()
        self.limit_backward = v_2
        return True

    def __r_vowel_suffix(self):
        v_1 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if not self.in_grouping_b(ItalianStemmer.g_AEIO):
                self.cursor = self.limit - v_1
                raise lab0()
            self.bra = self.cursor
            if not self.__r_RV():
                self.cursor = self.limit - v_1
                raise lab0()
            self.slice_del()
            self.ket = self.cursor
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "i":
                self.cursor = self.limit - v_1
                raise lab0()
            self.cursor -= 1
            self.bra = self.cursor
            if not self.__r_RV():
                self.cursor = self.limit - v_1
                raise lab0()
            self.slice_del()
        except lab0: pass
        v_2 = self.limit - self.cursor
        try:
            self.ket = self.cursor
            if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "h":
                self.cursor = self.limit - v_2
                raise lab0()
            self.cursor -= 1
            self.bra = self.cursor
            if not self.in_grouping_b(ItalianStemmer.g_CG):
                self.cursor = self.limit - v_2
                raise lab0()
            if not self.__r_RV():
                self.cursor = self.limit - v_2
                raise lab0()
            self.slice_del()
        except lab0: pass
        return True

    def _stem(self):
        v_1 = self.cursor
        self.__r_elisions()
        self.cursor = v_1
        v_2 = self.cursor
        self.__r_prelude()
        self.cursor = v_2
        self.__r_mark_regions()
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_3 = self.limit - self.cursor
        self.__r_attached_pronoun()
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        try:
            while True:
                v_5 = self.limit - self.cursor
                try:
                    if not self.__r_standard_suffix():
                        raise lab1()
                    break
                except lab1: pass
                self.cursor = self.limit - v_5
                if not self.__r_verb_suffix():
                    raise lab0()
                break
        except lab0: pass
        self.cursor = self.limit - v_4
        v_6 = self.limit - self.cursor
        self.__r_vowel_suffix()
        self.cursor = self.limit - v_6
        self.cursor = self.limit_backward
        v_7 = self.cursor
        self.__r_postlude()
        self.cursor = v_7
        return True

    a_0 = [
        Among("all'", -1, -1),
        Among("d'", -1, -1),
        Among("dall'", -1, -1),
        Among("dell'", -1, -1),
        Among("gl'", -1, -1),
        Among("l'", -1, -1),
        Among("m'", -1, -1),
        Among("nell'", -1, -1),
        Among("quell'", -1, -1),
        Among("quest'", -1, -1),
        Among("s'", -1, -1),
        Among("sull'", -1, -1),
        Among("t'", -1, -1),
        Among("tutt'", -1, -1),
        Among("un'", -1, -1),
        Among("v'", -1, -1)
    ]

    a_1 = [
        Among("", -1, 7),
        Among("qu", 0, 6),
        Among("á", 0, 1),
        Among("é", 0, 2),
        Among("í", 0, 3),
        Among("ó", 0, 4),
        Among("ú", 0, 5)
    ]

    a_2 = [
        Among("", -1, 3),
        Among("I", 0, 1),
        Among("U", 0, 2)
    ]

    a_3 = [
        Among("la", -1, -1),
        Among("cela", 0, -1),
        Among("gliela", 0, -1),
        Among("mela", 0, -1),
        Among("tela", 0, -1),
        Among("vela", 0, -1),
        Among("le", -1, -1),
        Among("cele", 6, -1),
        Among("gliele", 6, -1),
        Among("mele", 6, -1),
        Among("tele", 6, -1),
        Among("vele", 6, -1),
        Among("ne", -1, -1),
        Among("cene", 12, -1),
        Among("gliene", 12, -1),
        Among("mene", 12, -1),
        Among("sene", 12, -1),
        Among("tene", 12, -1),
        Among("vene", 12, -1),
        Among("ci", -1, -1),
        Among("li", -1, -1),
        Among("celi", 20, -1),
        Among("glieli", 20, -1),
        Among("meli", 20, -1),
        Among("teli", 20, -1),
        Among("veli", 20, -1),
        Among("gli", 20, -1),
        Among("mi", -1, -1),
        Among("si", -1, -1),
        Among("ti", -1, -1),
        Among("vi", -1, -1),
        Among("lo", -1, -1),
        Among("celo", 31, -1),
        Among("glielo", 31, -1),
        Among("melo", 31, -1),
        Among("telo", 31, -1),
        Among("velo", 31, -1)
    ]

    a_4 = [
        Among("ando", -1, 1),
        Among("endo", -1, 1),
        Among("ar", -1, 2),
        Among("er", -1, 2),
        Among("ir", -1, 2)
    ]

    a_5 = [
        Among("ic", -1, -1),
        Among("abil", -1, -1),
        Among("os", -1, -1),
        Among("iv", -1, 1)
    ]

    a_6 = [
        Among("ic", -1, 1),
        Among("abil", -1, 1),
        Among("iv", -1, 1)
    ]

    a_7 = [
        Among("ica", -1, 1),
        Among("logia", -1, 3),
        Among("osa", -1, 1),
        Among("ista", -1, 1),
        Among("iva", -1, 9),
        Among("anza", -1, 1),
        Among("enza", -1, 5),
        Among("ice", -1, 1),
        Among("atrice", 7, 1),
        Among("iche", -1, 1),
        Among("logie", -1, 3),
        Among("abile", -1, 1),
        Among("ibile", -1, 1),
        Among("usione", -1, 4),
        Among("azione", -1, 2),
        Among("uzione", -1, 4),
        Among("atore", -1, 2),
        Among("ose", -1, 1),
        Among("ante", -1, 1),
        Among("mente", -1, 1),
        Among("amente", 19, 7),
        Among("iste", -1, 1),
        Among("ive", -1, 9),
        Among("anze", -1, 1),
        Among("enze", -1, 5),
        Among("ici", -1, 1),
        Among("atrici", 25, 1),
        Among("ichi", -1, 1),
        Among("abili", -1, 1),
        Among("ibili", -1, 1),
        Among("ismi", -1, 1),
        Among("usioni", -1, 4),
        Among("azioni", -1, 2),
        Among("uzioni", -1, 4),
        Among("atori", -1, 2),
        Among("osi", -1, 1),
        Among("anti", -1, 1),
        Among("amenti", -1, 6),
        Among("imenti", -1, 6),
        Among("isti", -1, 1),
        Among("ivi", -1, 9),
        Among("ico", -1, 1),
        Among("ismo", -1, 1),
        Among("oso", -1, 1),
        Among("amento", -1, 6),
        Among("imento", -1, 6),
        Among("ivo", -1, 9),
        Among("ità", -1, 8),
        Among("istà", -1, 1),
        Among("istè", -1, 1),
        Among("istì", -1, 1)
    ]

    a_8 = [
        Among("isca", -1, 1),
        Among("enda", -1, 1),
        Among("ata", -1, 1),
        Among("ita", -1, 1),
        Among("uta", -1, 1),
        Among("ava", -1, 1),
        Among("eva", -1, 1),
        Among("iva", -1, 1),
        Among("erebbe", -1, 1),
        Among("irebbe", -1, 1),
        Among("isce", -1, 1),
        Among("ende", -1, 1),
        Among("are", -1, 1),
        Among("ere", -1, 1),
        Among("ire", -1, 1),
        Among("asse", -1, 1),
        Among("ate", -1, 1),
        Among("avate", 16, 1),
        Among("evate", 16, 1),
        Among("ivate", 16, 1),
        Among("ete", -1, 1),
        Among("erete", 20, 1),
        Among("irete", 20, 1),
        Among("ite", -1, 1),
        Among("ereste", -1, 1),
        Among("ireste", -1, 1),
        Among("ute", -1, 1),
        Among("erai", -1, 1),
        Among("irai", -1, 1),
        Among("isci", -1, 1),
        Among("endi", -1, 1),
        Among("erei", -1, 1),
        Among("irei", -1, 1),
        Among("assi", -1, 1),
        Among("ati", -1, 1),
        Among("iti", -1, 1),
        Among("eresti", -1, 1),
        Among("iresti", -1, 1),
        Among("uti", -1, 1),
        Among("avi", -1, 1),
        Among("evi", -1, 1),
        Among("ivi", -1, 1),
        Among("isco", -1, 1),
        Among("ando", -1, 1),
        Among("endo", -1, 1),
        Among("Yamo", -1, 1),
        Among("iamo", -1, 1),
        Among("avamo", -1, 1),
        Among("evamo", -1, 1),
        Among("ivamo", -1, 1),
        Among("eremo", -1, 1),
        Among("iremo", -1, 1),
        Among("assimo", -1, 1),
        Among("ammo", -1, 1),
        Among("emmo", -1, 1),
        Among("eremmo", 54, 1),
        Among("iremmo", 54, 1),
        Among("immo", -1, 1),
        Among("ano", -1, 1),
        Among("iscano", 58, 1),
        Among("avano", 58, 1),
        Among("evano", 58, 1),
        Among("ivano", 58, 1),
        Among("eranno", -1, 1),
        Among("iranno", -1, 1),
        Among("ono", -1, 1),
        Among("iscono", 65, 1),
        Among("arono", 65, 1),
        Among("erono", 65, 1),
        Among("irono", 65, 1),
        Among("erebbero", -1, 1),
        Among("irebbero", -1, 1),
        Among("assero", -1, 1),
        Among("essero", -1, 1),
        Among("issero", -1, 1),
        Among("ato", -1, 1),
        Among("ito", -1, 1),
        Among("uto", -1, 1),
        Among("avo", -1, 1),
        Among("evo", -1, 1),
        Among("ivo", -1, 1),
        Among("ar", -1, 1),
        Among("ir", -1, 1),
        Among("erà", -1, 1),
        Among("irà", -1, 1),
        Among("erò", -1, 1),
        Among("irò", -1, 1)
    ]


class lab0(BaseException): pass


class lab1(BaseException): pass


class lab2(BaseException): pass



__all__ = ('language', 'stemmer')

try:
    import Stemmer
    algorithms = Stemmer.algorithms
    stemmer = Stemmer.Stemmer
except ImportError:
    from .arabic_stemmer import ArabicStemmer
    from .armenian_stemmer import ArmenianStemmer
    from .basque_stemmer import BasqueStemmer
    from .catalan_stemmer import CatalanStemmer
    from .czech_stemmer import CzechStemmer
    from .danish_stemmer import DanishStemmer
    from .dutch_porter_stemmer import DutchPorterStemmer
    from .dutch_stemmer import DutchStemmer
    from .english_stemmer import EnglishStemmer
    from .esperanto_stemmer import EsperantoStemmer
    from .estonian_stemmer import EstonianStemmer
    from .finnish_stemmer import FinnishStemmer
    from .french_stemmer import FrenchStemmer
    from .german_stemmer import GermanStemmer
    from .greek_stemmer import GreekStemmer
    from .hindi_stemmer import HindiStemmer
    from .hungarian_stemmer import HungarianStemmer
    from .indonesian_stemmer import IndonesianStemmer
    from .irish_stemmer import IrishStemmer
    from .italian_stemmer import ItalianStemmer
    from .lithuanian_stemmer import LithuanianStemmer
    from .nepali_stemmer import NepaliStemmer
    from .norwegian_stemmer import NorwegianStemmer
    from .persian_stemmer import PersianStemmer
    from .polish_stemmer import PolishStemmer
    from .porter_stemmer import PorterStemmer
    from .portuguese_stemmer import PortugueseStemmer
    from .romanian_stemmer import RomanianStemmer
    from .russian_stemmer import RussianStemmer
    from .serbian_stemmer import SerbianStemmer
    from .sesotho_stemmer import SesothoStemmer
    from .spanish_stemmer import SpanishStemmer
    from .swedish_stemmer import SwedishStemmer
    from .tamil_stemmer import TamilStemmer
    from .turkish_stemmer import TurkishStemmer
    from .yiddish_stemmer import YiddishStemmer

    _languages = {
        'arabic': ArabicStemmer,
        'armenian': ArmenianStemmer,
        'basque': BasqueStemmer,
        'catalan': CatalanStemmer,
        'czech': CzechStemmer,
        'danish': DanishStemmer,
        'dutch': DutchStemmer,
        'dutch_porter': DutchPorterStemmer,
        'english': EnglishStemmer,
        'esperanto': EsperantoStemmer,
        'estonian': EstonianStemmer,
        'finnish': FinnishStemmer,
        'french': FrenchStemmer,
        'german': GermanStemmer,
        'greek': GreekStemmer,
        'hindi': HindiStemmer,
        'hungarian': HungarianStemmer,
        'indonesian': IndonesianStemmer,
        'irish': IrishStemmer,
        'italian': ItalianStemmer,
        'lithuanian': LithuanianStemmer,
        'nepali': NepaliStemmer,
        'norwegian': NorwegianStemmer,
        'persian': PersianStemmer,
        'polish': PolishStemmer,
        'porter': PorterStemmer,
        'portuguese': PortugueseStemmer,
        'romanian': RomanianStemmer,
        'russian': RussianStemmer,
        'serbian': SerbianStemmer,
        'sesotho': SesothoStemmer,
        'spanish': SpanishStemmer,
        'swedish': SwedishStemmer,
        'tamil': TamilStemmer,
        'turkish': TurkishStemmer,
        'yiddish': YiddishStemmer,
    }

    def algorithms():
        return list(_languages.keys())

    def stemmer(lang):
        lang = lang.lower()
        if lang in _languages:
            return _languages[lang]()
        else:
            raise KeyError("Stemming algorithm '%s' not found" % lang)



class Among:
    def __init__(self, s, substring_i, result, method=None):
        """
        @ivar s search string
        @ivar substring index to longest matching substring
        @ivar result of the lookup
        @ivar method method to use if substring matches
        """
        self.s = s
        self.substring_i = substring_i
        self.result = result
        self.method = method



# Generated from sesotho.sbl by Snowball 3.1.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class SesothoStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from sesotho.sbl by Snowball 3.1.0 - https://snowballstem.org/
    '''

    g_v = {"a", "e", "i", "o", "u"}

    I_pV = 0

    def __r_mark_regions(self):
        v_1 = self.cursor
        if not self.go_out_grouping(SesothoStemmer.g_v):
            return False
        self.cursor += 1
        self.I_pV = self.cursor
        self.cursor = v_1
        v_2 = self.cursor
        if self.cursor + 2 > self.limit:
            return False
        self.cursor += 2
        try:
            if self.cursor <= self.I_pV:
                raise lab0()
            self.I_pV = self.cursor
        except lab0: pass
        self.cursor = v_2
        return True

    def __r_remove_noun_prefixes(self):
        self.bra = self.cursor
        if self.find_among(SesothoStemmer.a_0) == 0:
            return False
        self.ket = self.cursor
        v_1 = self.cursor
        if self.cursor >= self.limit:
            return False
        self.cursor += 1
        try:
            if self.cursor < self.limit:
                raise lab0()
            return False
        except lab0: pass
        self.cursor = v_1
        if not self.go_out_grouping(SesothoStemmer.g_v):
            return False
        self.cursor += 1
        self.slice_del()
        return True

    def __r_remove_verb_suffixes(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        if self.find_among_b(SesothoStemmer.a_1) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.slice_del()
        self.limit_backward = v_2
        return True

    def __r_remove_nominal_suffixes(self):
        if self.cursor < self.I_pV:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_pV
        self.ket = self.cursor
        if self.find_among_b(SesothoStemmer.a_2) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.slice_del()
        self.limit_backward = v_2
        return True

    def _stem(self):
        if not self.__r_mark_regions():
            return False
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_1 = self.limit - self.cursor
        self.__r_remove_nominal_suffixes()
        self.cursor = self.limit - v_1
        v_2 = self.limit - self.cursor
        self.__r_remove_verb_suffixes()
        self.cursor = self.limit - v_2
        self.cursor = self.limit_backward
        v_3 = self.cursor
        self.__r_remove_noun_prefixes()
        self.cursor = v_3
        return True

    a_0 = [
        Among("ba", -1, -1),
        Among("boi", -1, -1),
        Among("le", -1, -1),
        Among("li", -1, -1),
        Among("ma", -1, -1),
        Among("me", -1, -1),
        Among("mo", -1, -1),
        Among("se", -1, -1)
    ]

    a_1 = [
        Among("a", -1, 1),
        Among("ela", 0, 1),
        Among("isa", 0, 1),
        Among("wa", 0, 1),
        Among("ile", -1, 1),
        Among("etse", -1, 1),
        Among("ang", -1, 1),
        Among("eng", -1, 1),
        Among("ong", -1, 1)
    ]

    a_2 = [
        Among("ana", -1, 1),
        Among("nyana", 0, 1),
        Among("oa", -1, 1),
        Among("i", -1, 1),
        Among("ano", -1, 1)
    ]


class lab0(BaseException): pass



# Generated from estonian.sbl by Snowball 3.1.0 - https://snowballstem.org/

from .basestemmer import BaseStemmer
from .among import Among


class EstonianStemmer(BaseStemmer):
    '''
    This class implements the stemming algorithm defined by a snowball script.
    Generated from estonian.sbl by Snowball 3.1.0 - https://snowballstem.org/
    '''

    g_V1 = {"a", "e", "i", "o", "u", "ä", "õ", "ö", "ü"}

    g_RV = {"'", "a", "e", "i", "o", "u"}

    g_KI = {"b", "d", "f", "g", "h", "k", "p", "s", "t", "z", "š", "ž"}

    g_GI = {"a", "c", "e", "i", "j", "l", "m", "n", "o", "q", "r", "u", "v", "w", "x", "ä", "õ", "ö", "ü"}

    I_p1 = 0

    def __r_mark_regions(self):
        self.I_p1 = self.limit
        while True:
            v_1 = self.cursor
            try:
                if self.cursor + 2 > self.limit:
                    raise lab0()
                self.cursor += 2
                while True:
                    try:
                        if self.cursor == self.limit or self.current[self.cursor] != "'":
                            raise lab1()
                        self.cursor += 1
                        break
                    except lab1: pass
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                break
            except lab0: pass
            self.cursor = v_1
            if not self.go_out_grouping(EstonianStemmer.g_V1):
                return False
            self.cursor += 1
            if not self.go_in_grouping(EstonianStemmer.g_V1):
                return False
            self.cursor += 1
            break
        self.I_p1 = self.cursor
        return True

    def __r_emphasis(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(EstonianStemmer.a_0)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        v_3 = self.limit - self.cursor
        if self.cursor - 4 < self.limit_backward:
            return False
        self.cursor -= 4
        self.cursor = self.limit - v_3
        if among_var == 1:
            v_4 = self.limit - self.cursor
            if not self.in_grouping_b(EstonianStemmer.g_GI):
                return False
            self.cursor = self.limit - v_4
            v_5 = self.limit - self.cursor
            try:
                if not self.__r_LONGV():
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_5
            self.slice_del()
        else:
            if not self.in_grouping_b(EstonianStemmer.g_KI):
                return False
            self.slice_del()
        return True

    def __r_verb(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(EstonianStemmer.a_1)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            self.slice_del()
        elif among_var == 2:
            self.slice_from("a")
        else:
            if not self.in_grouping_b(EstonianStemmer.g_V1):
                return False
            self.slice_del()
        return True

    def __r_LONGV(self):
        return self.find_among_b(EstonianStemmer.a_2) != 0

    def __r_i_plural(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "i":
            self.limit_backward = v_2
            return False
        self.cursor -= 1
        self.bra = self.cursor
        self.limit_backward = v_2
        if not self.in_grouping_b(EstonianStemmer.g_RV):
            return False
        self.slice_del()
        return True

    def __r_special_noun_endings(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(EstonianStemmer.a_3)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        self.slice_from(EstonianStemmer.as_3[among_var - 1])
        return True

    def __r_case_ending(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(EstonianStemmer.a_4)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            while True:
                try:
                    if not self.in_grouping_b(EstonianStemmer.g_RV):
                        raise lab0()
                    break
                except lab0: pass
                if not self.__r_LONGV():
                    return False
                break
        else:
            v_3 = self.limit - self.cursor
            if self.cursor - 4 < self.limit_backward:
                return False
            self.cursor -= 4
            self.cursor = self.limit - v_3
        self.slice_del()
        return True

    def __r_plural_three_first_cases(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(EstonianStemmer.a_6)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            self.slice_from("iku")
        elif among_var == 2:
            v_3 = self.limit - self.cursor
            try:
                if not self.__r_LONGV():
                    raise lab0()
                return False
            except lab0: pass
            self.cursor = self.limit - v_3
            self.slice_del()
        elif among_var == 3:
            while True:
                v_4 = self.limit - self.cursor
                try:
                    v_5 = self.limit - self.cursor
                    if self.cursor - 4 < self.limit_backward:
                        raise lab0()
                    self.cursor -= 4
                    self.cursor = self.limit - v_5
                    among_var = self.find_among_b(EstonianStemmer.a_5)
                    if among_var == 1:
                        self.slice_from("e")
                    elif among_var == 2:
                        self.slice_del()
                    break
                except lab0: pass
                self.cursor = self.limit - v_4
                self.slice_from("t")
                break
        else:
            while True:
                try:
                    if not self.in_grouping_b(EstonianStemmer.g_RV):
                        raise lab0()
                    break
                except lab0: pass
                if not self.__r_LONGV():
                    return False
                break
            self.slice_del()
        return True

    def __r_nu(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        if self.find_among_b(EstonianStemmer.a_7) == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        self.slice_del()
        return True

    def __r_undouble_kpt(self):
        if not self.in_grouping_b(EstonianStemmer.g_V1):
            return False
        if self.I_p1 > self.cursor:
            return False
        self.ket = self.cursor
        among_var = self.find_among_b(EstonianStemmer.a_8)
        if among_var == 0:
            return False
        self.bra = self.cursor
        self.slice_from(EstonianStemmer.as_8[among_var - 1])
        return True

    def __r_degrees(self):
        if self.cursor < self.I_p1:
            return False
        v_2 = self.limit_backward
        self.limit_backward = self.I_p1
        self.ket = self.cursor
        among_var = self.find_among_b(EstonianStemmer.a_9)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        self.bra = self.cursor
        self.limit_backward = v_2
        if among_var == 1:
            if not self.in_grouping_b(EstonianStemmer.g_RV):
                return False
            self.slice_del()
        else:
            self.slice_del()
        return True

    def __r_substantive(self):
        v_1 = self.limit - self.cursor
        self.__r_special_noun_endings()
        self.cursor = self.limit - v_1
        v_2 = self.limit - self.cursor
        self.__r_case_ending()
        self.cursor = self.limit - v_2
        v_3 = self.limit - self.cursor
        self.__r_plural_three_first_cases()
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        self.__r_degrees()
        self.cursor = self.limit - v_4
        v_5 = self.limit - self.cursor
        self.__r_i_plural()
        self.cursor = self.limit - v_5
        v_6 = self.limit - self.cursor
        self.__r_nu()
        self.cursor = self.limit - v_6
        return True

    def __r_verb_exceptions(self):
        self.bra = self.cursor
        among_var = self.find_among(EstonianStemmer.a_10)
        if among_var == 0:
            return False
        self.ket = self.cursor
        if self.cursor < self.limit:
            return False
        self.slice_from(EstonianStemmer.as_10[among_var - 1])
        return True

    def _stem(self):
        v_1 = self.cursor
        try:
            if not self.__r_verb_exceptions():
                raise lab0()
            return False
        except lab0: pass
        self.cursor = v_1
        v_2 = self.cursor
        self.__r_mark_regions()
        self.cursor = v_2
        self.limit_backward = self.cursor
        self.cursor = self.limit
        v_3 = self.limit - self.cursor
        self.__r_emphasis()
        self.cursor = self.limit - v_3
        v_4 = self.limit - self.cursor
        try:
            while True:
                v_5 = self.limit - self.cursor
                try:
                    if not self.__r_verb():
                        raise lab1()
                    break
                except lab1: pass
                self.cursor = self.limit - v_5
                self.__r_substantive()
                break
        except lab0: pass
        self.cursor = self.limit - v_4
        v_6 = self.limit - self.cursor
        self.__r_undouble_kpt()
        self.cursor = self.limit - v_6
        self.ket = self.cursor
        if self.cursor <= self.limit_backward or self.current[self.cursor - 1] != "'":
            return False
        self.cursor -= 1
        self.bra = self.cursor
        self.slice_del()
        self.cursor = self.limit_backward
        return True

    a_0 = [
        Among("gi", -1, 1),
        Among("ki", -1, 2)
    ]

    a_1 = [
        Among("da", -1, 3),
        Among("mata", -1, 1),
        Among("b", -1, 3),
        Among("ksid", -1, 1),
        Among("nuksid", 3, 1),
        Among("me", -1, 3),
        Among("sime", 5, 1),
        Among("ksime", 6, 1),
        Among("nuksime", 7, 1),
        Among("akse", -1, 2),
        Among("dakse", 9, 1),
        Among("takse", 9, 1),
        Among("site", -1, 1),
        Among("ksite", 12, 1),
        Among("nuksite", 13, 1),
        Among("n", -1, 3),
        Among("sin", 15, 1),
        Among("ksin", 16, 1),
        Among("nuksin", 17, 1),
        Among("daks", -1, 1),
        Among("taks", -1, 1)
    ]

    a_2 = [
        Among("aa", -1, -1),
        Among("ee", -1, -1),
        Among("ii", -1, -1),
        Among("oo", -1, -1),
        Among("uu", -1, -1),
        Among("ää", -1, -1),
        Among("õõ", -1, -1),
        Among("öö", -1, -1),
        Among("üü", -1, -1)
    ]

    a_3 = [
        Among("lane", -1, 1),
        Among("line", -1, 3),
        Among("mine", -1, 2),
        Among("lasse", -1, 1),
        Among("lisse", -1, 3),
        Among("misse", -1, 2),
        Among("lasi", -1, 1),
        Among("lisi", -1, 3),
        Among("misi", -1, 2),
        Among("last", -1, 1),
        Among("list", -1, 3),
        Among("mist", -1, 2)
    ]
    as_3 = ("lase", "mise", "lise")

    a_4 = [
        Among("ga", -1, 1),
        Among("ta", -1, 1),
        Among("le", -1, 1),
        Among("sse", -1, 1),
        Among("l", -1, 1),
        Among("s", -1, 1),
        Among("ks", 5, 1),
        Among("t", -1, 2),
        Among("lt", 7, 1),
        Among("st", 7, 1)
    ]

    a_5 = [
        Among("", -1, 2),
        Among("las", 0, 1),
        Among("lis", 0, 1),
        Among("mis", 0, 1),
        Among("t", 0, -1)
    ]

    a_6 = [
        Among("d", -1, 4),
        Among("sid", 0, 2),
        Among("de", -1, 4),
        Among("ikkude", 2, 1),
        Among("ike", -1, 1),
        Among("ikke", -1, 1),
        Among("te", -1, 3)
    ]

    a_7 = [
        Among("va", -1, -1),
        Among("du", -1, -1),
        Among("nu", -1, -1),
        Among("tu", -1, -1)
    ]

    a_8 = [
        Among("kk", -1, 1),
        Among("pp", -1, 2),
        Among("tt", -1, 3)
    ]
    as_8 = ("k", "p", "t")

    a_9 = [
        Among("ma", -1, 2),
        Among("mai", -1, 1),
        Among("m", -1, 1)
    ]

    a_10 = [
        Among("joob", -1, 1),
        Among("jood", -1, 1),
        Among("joodakse", 1, 1),
        Among("jooma", -1, 1),
        Among("joomata", 3, 1),
        Among("joome", -1, 1),
        Among("joon", -1, 1),
        Among("joote", -1, 1),
        Among("joovad", -1, 1),
        Among("juua", -1, 1),
        Among("juuakse", 9, 1),
        Among("jäi", -1, 12),
        Among("jäid", 11, 12),
        Among("jäime", 11, 12),
        Among("jäin", 11, 12),
        Among("jäite", 11, 12),
        Among("jääb", -1, 12),
        Among("jääd", -1, 12),
        Among("jääda", 17, 12),
        Among("jäädakse", 18, 12),
        Among("jäädi", 17, 12),
        Among("jääks", -1, 12),
        Among("jääksid", 21, 12),
        Among("jääksime", 21, 12),
        Among("jääksin", 21, 12),
        Among("jääksite", 21, 12),
        Among("jääma", -1, 12),
        Among("jäämata", 26, 12),
        Among("jääme", -1, 12),
        Among("jään", -1, 12),
        Among("jääte", -1, 12),
        Among("jäävad", -1, 12),
        Among("jõi", -1, 1),
        Among("jõid", 32, 1),
        Among("jõime", 32, 1),
        Among("jõin", 32, 1),
        Among("jõite", 32, 1),
        Among("keeb", -1, 4),
        Among("keed", -1, 4),
        Among("keedakse", 38, 4),
        Among("keeks", -1, 4),
        Among("keeksid", 40, 4),
        Among("keeksime", 40, 4),
        Among("keeksin", 40, 4),
        Among("keeksite", 40, 4),
        Among("keema", -1, 4),
        Among("keemata", 45, 4),
        Among("keeme", -1, 4),
        Among("keen", -1, 4),
        Among("kees", -1, 4),
        Among("keeta", -1, 4),
        Among("keete", -1, 4),
        Among("keevad", -1, 4),
        Among("käia", -1, 8),
        Among("käiakse", 53, 8),
        Among("käib", -1, 8),
        Among("käid", -1, 8),
        Among("käidi", 56, 8),
        Among("käiks", -1, 8),
        Among("käiksid", 58, 8),
        Among("käiksime", 58, 8),
        Among("käiksin", 58, 8),
        Among("käiksite", 58, 8),
        Among("käima", -1, 8),
        Among("käimata", 63, 8),
        Among("käime", -1, 8),
        Among("käin", -1, 8),
        Among("käis", -1, 8),
        Among("käite", -1, 8),
        Among("käivad", -1, 8),
        Among("laob", -1, 16),
        Among("laod", -1, 16),
        Among("laoks", -1, 16),
        Among("laoksid", 72, 16),
        Among("laoksime", 72, 16),
        Among("laoksin", 72, 16),
        Among("laoksite", 72, 16),
        Among("laome", -1, 16),
        Among("laon", -1, 16),
        Among("laote", -1, 16),
        Among("laovad", -1, 16),
        Among("loeb", -1, 14),
        Among("loed", -1, 14),
        Among("loeks", -1, 14),
        Among("loeksid", 83, 14),
        Among("loeksime", 83, 14),
        Among("loeksin", 83, 14),
        Among("loeksite", 83, 14),
        Among("loeme", -1, 14),
        Among("loen", -1, 14),
        Among("loete", -1, 14),
        Among("loevad", -1, 14),
        Among("loob", -1, 7),
        Among("lood", -1, 7),
        Among("loodi", 93, 7),
        Among("looks", -1, 7),
        Among("looksid", 95, 7),
        Among("looksime", 95, 7),
        Among("looksin", 95, 7),
        Among("looksite", 95, 7),
        Among("looma", -1, 7),
        Among("loomata", 100, 7),
        Among("loome", -1, 7),
        Among("loon", -1, 7),
        Among("loote", -1, 7),
        Among("loovad", -1, 7),
        Among("luua", -1, 7),
        Among("luuakse", 106, 7),
        Among("lõi", -1, 6),
        Among("lõid", 108, 6),
        Among("lõime", 108, 6),
        Among("lõin", 108, 6),
        Among("lõite", 108, 6),
        Among("lööb", -1, 5),
        Among("lööd", -1, 5),
        Among("löödakse", 114, 5),
        Among("löödi", 114, 5),
        Among("lööks", -1, 5),
        Among("lööksid", 117, 5),
        Among("lööksime", 117, 5),
        Among("lööksin", 117, 5),
        Among("lööksite", 117, 5),
        Among("lööma", -1, 5),
        Among("löömata", 122, 5),
        Among("lööme", -1, 5),
        Among("löön", -1, 5),
        Among("lööte", -1, 5),
        Among("löövad", -1, 5),
        Among("lüüa", -1, 5),
        Among("lüüakse", 128, 5),
        Among("müüa", -1, 13),
        Among("müüakse", 130, 13),
        Among("müüb", -1, 13),
        Among("müüd", -1, 13),
        Among("müüdi", 133, 13),
        Among("müüks", -1, 13),
        Among("müüksid", 135, 13),
        Among("müüksime", 135, 13),
        Among("müüksin", 135, 13),
        Among("müüksite", 135, 13),
        Among("müüma", -1, 13),
        Among("müümata", 140, 13),
        Among("müüme", -1, 13),
        Among("müün", -1, 13),
        Among("müüs", -1, 13),
        Among("müüte", -1, 13),
        Among("müüvad", -1, 13),
        Among("näeb", -1, 18),
        Among("näed", -1, 18),
        Among("näeks", -1, 18),
        Among("näeksid", 149, 18),
        Among("näeksime", 149, 18),
        Among("näeksin", 149, 18),
        Among("näeksite", 149, 18),
        Among("näeme", -1, 18),
        Among("näen", -1, 18),
        Among("näete", -1, 18),
        Among("näevad", -1, 18),
        Among("nägema", -1, 18),
        Among("nägemata", 158, 18),
        Among("näha", -1, 18),
        Among("nähakse", 160, 18),
        Among("nähti", -1, 18),
        Among("põeb", -1, 15),
        Among("põed", -1, 15),
        Among("põeks", -1, 15),
        Among("põeksid", 165, 15),
        Among("põeksime", 165, 15),
        Among("põeksin", 165, 15),
        Among("põeksite", 165, 15),
        Among("põeme", -1, 15),
        Among("põen", -1, 15),
        Among("põete", -1, 15),
        Among("põevad", -1, 15),
        Among("saab", -1, 2),
        Among("saad", -1, 2),
        Among("saada", 175, 2),
        Among("saadakse", 176, 2),
        Among("saadi", 175, 2),
        Among("saaks", -1, 2),
        Among("saaksid", 179, 2),
        Among("saaksime", 179, 2),
        Among("saaksin", 179, 2),
        Among("saaksite", 179, 2),
        Among("saama", -1, 2),
        Among("saamata", 184, 2),
        Among("saame", -1, 2),
        Among("saan", -1, 2),
        Among("saate", -1, 2),
        Among("saavad", -1, 2),
        Among("sai", -1, 2),
        Among("said", 190, 2),
        Among("saime", 190, 2),
        Among("sain", 190, 2),
        Among("saite", 190, 2),
        Among("sõi", -1, 9),
        Among("sõid", 195, 9),
        Among("sõime", 195, 9),
        Among("sõin", 195, 9),
        Among("sõite", 195, 9),
        Among("sööb", -1, 9),
        Among("sööd", -1, 9),
        Among("söödakse", 201, 9),
        Among("söödi", 201, 9),
        Among("sööks", -1, 9),
        Among("sööksid", 204, 9),
        Among("sööksime", 204, 9),
        Among("sööksin", 204, 9),
        Among("sööksite", 204, 9),
        Among("sööma", -1, 9),
        Among("söömata", 209, 9),
        Among("sööme", -1, 9),
        Among("söön", -1, 9),
        Among("sööte", -1, 9),
        Among("söövad", -1, 9),
        Among("süüa", -1, 9),
        Among("süüakse", 215, 9),
        Among("teeb", -1, 17),
        Among("teed", -1, 17),
        Among("teeks", -1, 17),
        Among("teeksid", 219, 17),
        Among("teeksime", 219, 17),
        Among("teeksin", 219, 17),
        Among("teeksite", 219, 17),
        Among("teeme", -1, 17),
        Among("teen", -1, 17),
        Among("teete", -1, 17),
        Among("teevad", -1, 17),
        Among("tegema", -1, 17),
        Among("tegemata", 228, 17),
        Among("teha", -1, 17),
        Among("tehakse", 230, 17),
        Among("tehti", -1, 17),
        Among("toob", -1, 10),
        Among("tood", -1, 10),
        Among("toodi", 234, 10),
        Among("tooks", -1, 10),
        Among("tooksid", 236, 10),
        Among("tooksime", 236, 10),
        Among("tooksin", 236, 10),
        Among("tooksite", 236, 10),
        Among("tooma", -1, 10),
        Among("toomata", 241, 10),
        Among("toome", -1, 10),
        Among("toon", -1, 10),
        Among("toote", -1, 10),
        Among("toovad", -1, 10),
        Among("tuua", -1, 10),
        Among("tuuakse", 247, 10),
        Among("tõi", -1, 10),
        Among("tõid", 249, 10),
        Among("tõime", 249, 10),
        Among("tõin", 249, 10),
        Among("tõite", 249, 10),
        Among("viia", -1, 3),
        Among("viiakse", 254, 3),
        Among("viib", -1, 3),
        Among("viid", -1, 3),
        Among("viidi", 257, 3),
        Among("viiks", -1, 3),
        Among("viiksid", 259, 3),
        Among("viiksime", 259, 3),
        Among("viiksin", 259, 3),
        Among("viiksite", 259, 3),
        Among("viima", -1, 3),
        Among("viimata", 264, 3),
        Among("viime", -1, 3),
        Among("viin", -1, 3),
        Among("viisime", -1, 3),
        Among("viisin", -1, 3),
        Among("viisite", -1, 3),
        Among("viite", -1, 3),
        Among("viivad", -1, 3),
        Among("võib", -1, 11),
        Among("võid", -1, 11),
        Among("võida", 274, 11),
        Among("võidakse", 275, 11),
        Among("võidi", 274, 11),
        Among("võiks", -1, 11),
        Among("võiksid", 278, 11),
        Among("võiksime", 278, 11),
        Among("võiksin", 278, 11),
        Among("võiksite", 278, 11),
        Among("võima", -1, 11),
        Among("võimata", 283, 11),
        Among("võime", -1, 11),
        Among("võin", -1, 11),
        Among("võis", -1, 11),
        Among("võite", -1, 11),
        Among("võivad", -1, 11)
    ]
    as_10 = ("joo", "saa", "viima", "keesi", "löö", "lõi", "loo", "käisi", "söö", "too", "võisi", "jääma", "müüsi", "luge", "põde", "ladu", "tegi", "nägi")


class lab0(BaseException): pass


class lab1(BaseException): pass



class BaseStemmer:
    def __init__(self):
        self.set_current("")

    def set_current(self, value):
        '''
        Set the self.current string.
        '''
        self.current = value
        self.cursor = 0
        self.limit = len(self.current)
        self.limit_backward = 0
        self.bra = self.cursor
        self.ket = self.limit

    def get_current(self):
        '''
        Get the self.current string.
        '''
        return self.current

    def copy_from(self, other):
        self.current          = other.current
        self.cursor           = other.cursor
        self.limit            = other.limit
        self.limit_backward   = other.limit_backward
        self.bra              = other.bra
        self.ket              = other.ket

    def in_grouping(self, s):
        if self.cursor >= self.limit:
            return False
        if self.current[self.cursor] not in s:
            return False
        self.cursor += 1
        return True

    def go_in_grouping(self, s):
        while self.cursor < self.limit:
            if self.current[self.cursor] not in s:
                return True
            self.cursor += 1
        return False

    def in_grouping_b(self, s):
        if self.cursor <= self.limit_backward:
            return False
        if self.current[self.cursor - 1] not in s:
            return False
        self.cursor -= 1
        return True

    def go_in_grouping_b(self, s):
        while self.cursor > self.limit_backward:
            if self.current[self.cursor - 1] not in s:
                return True
            self.cursor -= 1
        return False

    def out_grouping(self, s):
        if self.cursor >= self.limit:
            return False
        if self.current[self.cursor] not in s:
            self.cursor += 1
            return True
        return False

    def go_out_grouping(self, s):
        while self.cursor < self.limit:
            if self.current[self.cursor] in s:
                return True
            self.cursor += 1
        return False

    def out_grouping_b(self, s):
        if self.cursor <= self.limit_backward:
            return False
        if self.current[self.cursor - 1] not in s:
            self.cursor -= 1
            return True
        return False

    def go_out_grouping_b(self, s):
        while self.cursor > self.limit_backward:
            if self.current[self.cursor - 1] in s:
                return True
            self.cursor -= 1
        return False

    def eq_s(self, s):
        if self.current.startswith(s, self.cursor, self.limit):
            self.cursor += len(s)
            return True
        return False

    def eq_s_b(self, s):
        if self.current.endswith(s, self.limit_backward, self.cursor):
            self.cursor -= len(s)
            return True
        return False

    def find_among(self, v):
        i = 0
        j = len(v)

        c = self.cursor
        l = self.limit

        common_i = 0
        common_j = 0

        first_key_inspected = False

        while True:
            k = i + ((j - i) >> 1)
            diff = 0
            common = min(common_i, common_j) # smaller
            w = v[k]
            for i2 in range(common, len(w.s)):
                if c + common == l:
                    diff = -1
                    break
                diff = ord(self.current[c + common]) - ord(w.s[i2])
                if diff != 0:
                    break
                common += 1
            if diff < 0:
                j = k
                common_j = common
            else:
                i = k
                common_i = common
            if j - i <= 1:
                if i > 0:
                    break # v->s has been inspected
                if j == i:
                    break # only one item in v
                # - but now we need to go round once more to get
                # v->s inspected. This looks messy, but is actually
                # the optimal approach.
                if first_key_inspected:
                    break
                first_key_inspected = True
        while True:
            w = v[i]
            if common_i >= len(w.s):
                self.cursor = c + len(w.s)
                if w.method is None:
                    return w.result
                if w.method(self):
                    self.cursor = c + len(w.s)
                    return w.result
            i = w.substring_i
            if i < 0:
                return 0
        return -1 # not reachable

    def find_among_b(self, v):
        '''
        find_among_b is for backwards processing. Same comments apply
        '''
        i = 0
        j = len(v)

        c = self.cursor
        lb = self.limit_backward

        common_i = 0
        common_j = 0

        first_key_inspected = False

        while True:
            k = i + ((j - i) >> 1)
            diff = 0
            common = min(common_i, common_j)
            w = v[k]
            for i2 in range(len(w.s) - 1 - common, -1, -1):
                if c - common == lb:
                    diff = -1
                    break
                diff = ord(self.current[c - 1 - common]) - ord(w.s[i2])
                if diff != 0:
                    break
                common += 1
            if diff < 0:
                j = k
                common_j = common
            else:
                i = k
                common_i = common
            if j - i <= 1:
                if i > 0:
                    break
                if j == i:
                    break
                if first_key_inspected:
                    break
                first_key_inspected = True
        while True:
            w = v[i]
            if common_i >= len(w.s):
                self.cursor = c - len(w.s)
                if w.method is None:
                    return w.result
                if w.method(self):
                    self.cursor = c - len(w.s)
                    return w.result
            i = w.substring_i
            if i < 0:
                return 0
        return -1 # not reachable

    def replace_s(self, c_bra, c_ket, s):
        '''
        to replace chars between c_bra and c_ket in self.current by the
        chars in s.

        @type c_bra int
        @type c_ket int
        @type s: string
        '''
        adjustment = len(s) - (c_ket - c_bra)
        self.current = self.current[0:c_bra] + s + self.current[c_ket:]
        self.limit += adjustment
        if self.cursor >= c_ket:
            self.cursor += adjustment
        elif self.cursor > c_bra:
            self.cursor = c_bra
        return adjustment

    def slice_from(self, s):
        '''
        @type s string
        '''
        assert self.bra >= 0
        assert self.bra <= self.ket
        assert self.ket <= self.limit
        assert self.limit <= len(self.current)
        self.replace_s(self.bra, self.ket, s)
        self.ket = self.bra + len(s)

    def slice_del(self):
        return self.slice_from("")

    def insert(self, c_bra, c_ket, s):
        '''
        @type c_bra int
        @type c_ket int
        @type s: string
        '''
        adjustment = self.replace_s(c_bra, c_ket, s)
        if c_bra <= self.bra:
            self.bra += adjustment
        if c_bra <= self.ket:
            self.ket += adjustment

    def slice_to(self):
        '''
        Return the slice as a string.
        '''
        assert self.bra >= 0
        assert self.bra <= self.ket
        assert self.ket <= self.limit
        assert self.limit <= len(self.current)
        return self.current[self.bra:self.ket]

    def assign_to(self):
        '''
        Return the current string up to the limit.
        '''
        return self.current[0:self.limit]

    def stemWord(self, word):
        self.set_current(word)
        self._stem()
        return self.get_current()

    def stemWords(self, words):
        return [self.stemWord(word) for word in words]
